@@ -1,9 +1,9 @@
 /* 
  * Main source module for the Citadel server
  *
- * Copyright (c) 1987-2010 by the citadel.org team
+ * Copyright (c) 1987-2011 by the citadel.org team
  *
- * This program is free software; you can redistribute it and/or modify
+ * This program is open source software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "sysdep.h"
@@ -82,6 +82,7 @@ char *unique_session_numbers;
 int ScheduledShutdown = 0;
 time_t server_startup_time;
 int panic_fd;
+int openid_level_supported = 0;
 
 /*
  * print the actual stack frame.
@@ -98,9 +99,9 @@ void cit_backtrace(void)
 	strings = backtrace_symbols(stack_frames, size);
 	for (i = 0; i < size; i++) {
 		if (strings != NULL)
-			CtdlLogPrintf(1, "%s\n", strings[i]);
+			syslog(LOG_ALERT, "%s\n", strings[i]);
 		else
-			CtdlLogPrintf(1, "%p\n", stack_frames[i]);
+			syslog(LOG_ALERT, "%p\n", stack_frames[i]);
 	}
 	free(strings);
 #endif
@@ -121,9 +122,9 @@ void cit_panic_backtrace(int SigNum)
 	strings = backtrace_symbols(stack_frames, size);
 	for (i = 0; i < size; i++) {
 		if (strings != NULL)
-			CtdlLogPrintf(1, "%s\n", strings[i]);
+			syslog(LOG_ALERT, "%s\n", strings[i]);
 		else
-			CtdlLogPrintf(1, "%p\n", stack_frames[i]);
+			syslog(LOG_ALERT, "%p\n", stack_frames[i]);
 	}
 	free(strings);
 #endif
@@ -140,19 +141,14 @@ void master_startup(void) {
 	struct ctdlroom qrbuf;
 	int rv;
 	
-	CtdlLogPrintf(CTDL_DEBUG, "master_startup() started\n");
+	syslog(LOG_DEBUG, "master_startup() started\n");
 	time(&server_startup_time);
 
-	CtdlLogPrintf(CTDL_INFO, "Opening databases\n");
+	syslog(LOG_INFO, "Opening databases\n");
 	open_databases();
-
-	ctdl_thread_internal_init_tsd();
-	
-	CtdlThreadAllocTSD();
-	
 	check_ref_counts();
 
-	CtdlLogPrintf(CTDL_INFO, "Creating base rooms (if necessary)\n");
+	syslog(LOG_INFO, "Creating base rooms (if necessary)\n");
 	CtdlCreateRoom(config.c_baseroom,	0, "", 0, 1, 0, VIEW_BBS);
 	CtdlCreateRoom(AIDEROOM,		3, "", 0, 1, 0, VIEW_BBS);
 	CtdlCreateRoom(SYSCONFIGROOM,		3, "", 0, 1, 0, VIEW_BBS);
@@ -170,10 +166,13 @@ void master_startup(void) {
                 CtdlPutRoomLock(&qrbuf);
         }
 
-	CtdlLogPrintf(CTDL_INFO, "Seeding the pseudo-random number generator...\n");
+	syslog(LOG_INFO, "Seeding the pseudo-random number generator...\n");
 	urandom = fopen("/dev/urandom", "r");
 	if (urandom != NULL) {
 		rv = fread(&seed, sizeof seed, 1, urandom);
+		if (rv == -1)
+			syslog(LOG_EMERG, "failed to read random seed: %s\n", 
+			       strerror(errno));
 		fclose(urandom);
 	}
 	else {
@@ -183,12 +182,12 @@ void master_startup(void) {
 	srand(seed);
 	srandom(seed);
 
-	CtdlLogPrintf(CTDL_INFO, "Initializing ipgm secret\n");
+	syslog(LOG_INFO, "Initializing ipgm secret\n");
 	get_config();
 	config.c_ipgm_secret = rand();
 	put_config();
 
-	CtdlLogPrintf(CTDL_DEBUG, "master_startup() finished\n");
+	syslog(LOG_DEBUG, "master_startup() finished\n");
 }
 
 
@@ -214,16 +213,12 @@ void master_cleanup(int exitcode) {
 	sysdep_master_cleanup();
 	
 	/* Close databases */
-	CtdlLogPrintf(CTDL_INFO, "Closing databases\n");
+	syslog(LOG_INFO, "Closing databases\n");
 	close_databases();
 
-#ifdef DEBUG_MEMORY_LEAKS
-	dump_heap();
-#endif
-
 	/* If the operator requested a halt but not an exit, halt here. */
 	if (shutdown_and_halt) {
-		CtdlLogPrintf(CTDL_NOTICE, "citserver: Halting server without exiting.\n");
+		syslog(LOG_NOTICE, "citserver: Halting server without exiting.\n");
 		fflush(stdout); fflush(stderr);
 		while(1) {
 			sleep(32767);
@@ -233,7 +228,7 @@ void master_cleanup(int exitcode) {
 	release_control();
 
 	/* Now go away. */
-	CtdlLogPrintf(CTDL_NOTICE, "citserver: Exiting with status %d\n", exitcode);
+	syslog(LOG_NOTICE, "citserver: Exiting with status %d\n", exitcode);
 	fflush(stdout); fflush(stderr);
 	
 	if (restart_server != 0)
@@ -262,7 +257,7 @@ void cmd_info(char *cmdbuf) {
 	cprintf("%s\n", config.c_moreprompt);
 	cprintf("1\n");	/* 1 = yes, this system supports floors */
 	cprintf("1\n"); /* 1 = we support the extended paging options */
-	cprintf("%s\n", CC->cs_nonce);
+	cprintf("\n");	/* nonce no longer supported */
 	cprintf("1\n"); /* 1 = yes, this system supports the QNOP command */
 
 #ifdef HAVE_LDAP
@@ -280,21 +275,23 @@ void cmd_info(char *cmdbuf) {
 
 	cprintf("%s\n", config.c_default_cal_zone);
 
-	/* Output load averages */
-	cprintf("%f\n", CtdlThreadLoadAvg);
-	cprintf("%f\n", CtdlThreadWorkerAvg);
-	cprintf("%d\n", CtdlThreadGetCount());
+	/* thread load averages -- temporarily disabled during refactoring of this code */
+	cprintf("0\n");		/* load average */
+	cprintf("0\n");		/* worker average */
+	cprintf("0\n");		/* thread count */
 
 	cprintf("1\n");		/* yes, Sieve mail filtering is supported */
 	cprintf("%d\n", config.c_enable_fulltext);
 	cprintf("%s\n", svn_revision());
 
 	if (config.c_auth_mode == AUTHMODE_NATIVE) {
-		cprintf("1\n");	/* OpenID is enabled when using native auth */
+		cprintf("%d\n", openid_level_supported); /* OpenID is enabled when using native auth */
 	}
 	else {
 		cprintf("0\n");	/* OpenID is disabled when using non-native auth */
 	}
+
+	cprintf("%d\n", config.c_guest_logins);
 	
 	cprintf("000\n");
 }
@@ -362,14 +359,14 @@ int is_public_client(void)
 	 */
 	if (stat(public_clients_file, &statbuf) != 0) {
 		/* No public_clients file exists, so bail out */
-		CtdlLogPrintf(CTDL_WARNING, "Warning: '%s' does not exist\n", 
+		syslog(LOG_WARNING, "Warning: '%s' does not exist\n", 
 				public_clients_file);
 		return(0);
 	}
 
 	if (statbuf.st_mtime > pc_timestamp) {
 		begin_critical_section(S_PUBLIC_CLIENTS);
-		CtdlLogPrintf(CTDL_INFO, "Loading %s\n", public_clients_file);
+		syslog(LOG_INFO, "Loading %s\n", public_clients_file);
 
 		public_clientspos = &public_clients[0];
 		public_clientsend = public_clientspos + SIZ;
@@ -415,18 +412,18 @@ int is_public_client(void)
 		end_critical_section(S_PUBLIC_CLIENTS);
 	}
 
-	CtdlLogPrintf(CTDL_DEBUG, "Checking whether %s is a local or public client\n",
+	syslog(LOG_DEBUG, "Checking whether %s is a local or public client\n",
 		CC->cs_addr);
 	for (i=0; i<num_parms(public_clients); ++i) {
 		extract_token(addrbuf, public_clients, i, '|', sizeof addrbuf);
 		if (!strcasecmp(CC->cs_addr, addrbuf)) {
-			CtdlLogPrintf(CTDL_DEBUG, "... yes it is.\n");
+			syslog(LOG_DEBUG, "... yes its local.\n");
 			return(1);
 		}
 	}
 
 	/* No hits.  This is not a public client. */
-	CtdlLogPrintf(CTDL_DEBUG, "... no it isn't.\n");
+	syslog(LOG_DEBUG, "... no it isn't.\n");
 	return(0);
 }
 
@@ -441,8 +438,6 @@ void cmd_iden(char *argbuf)
 	int rev_level;
 	char desc[128];
 	char from_host[128];
-	struct in_addr addr;
-	int do_lookup = 0;
 
 	if (num_parms(argbuf)<4) {
 		cprintf("%d usage error\n", ERROR + ILLEGAL_VALUE);
@@ -464,25 +459,14 @@ void cmd_iden(char *argbuf)
 	safestrncpy(CC->cs_clientname, desc, sizeof CC->cs_clientname);
 	CC->cs_clientname[31] = 0;
 
-	if (!IsEmptyStr(from_host)) {
-		if (CC->is_local_socket) do_lookup = 1;
-		else if (is_public_client()) do_lookup = 1;
+	/* For local sockets and public clients, trust the hostname supplied by the client */
+	if ( (CC->is_local_socket) || (is_public_client()) ) {
+		safestrncpy(CC->cs_host, from_host, sizeof CC->cs_host);
+		CC->cs_host[sizeof CC->cs_host - 1] = 0;
+		CC->cs_addr[0] = 0;
 	}
 
-	if (do_lookup) {
-		CtdlLogPrintf(CTDL_DEBUG, "Looking up hostname '%s'\n", from_host);
-		if ((addr.s_addr = inet_addr(from_host)) != -1) {
-			locate_host(CC->cs_host, sizeof CC->cs_host,
-				CC->cs_addr, sizeof CC->cs_addr,
-				&addr);
-		}
-	   	else {
-			safestrncpy(CC->cs_host, from_host, sizeof CC->cs_host);
-			CC->cs_host[sizeof CC->cs_host - 1] = 0;
-		}
-	}
-
-	CtdlLogPrintf(CTDL_NOTICE, "Client %d/%d/%01d.%02d (%s) from %s\n",
+	syslog(LOG_NOTICE, "Client %d/%d/%01d.%02d (%s) from %s\n",
 		dev_code,
 		cli_code,
 		(rev_level / 100),
@@ -645,7 +629,7 @@ void GenerateRoomDisplay(char *real_room,
 	if (viewed->room.QRflags & QR_PRIVATE) {
 		CtdlRoomAccess(&viewed->room, &viewer->user, &ra, NULL);
 		if ( (ra & UA_KNOWN) == 0) {
-			strcpy(real_room, "<private room>");
+			strcpy(real_room, " ");
 		}
 	}
 
@@ -670,6 +654,11 @@ int CtdlAccessCheck(int required_level) {
 		return(-1);
 	}
 
+	if ((required_level >= ac_logged_in_or_guest) && (CC->logged_in == 0) && (!config.c_guest_logins)) {
+		cprintf("%d Not logged in.\n", ERROR + NOT_LOGGED_IN);
+		return(-1);
+	}
+
 	if ((required_level >= ac_logged_in) && (CC->logged_in == 0)) {
 		cprintf("%d Not logged in.\n", ERROR + NOT_LOGGED_IN);
 		return(-1);
@@ -769,8 +758,8 @@ void cmd_ipgm(char *argbuf)
 	else {
 		sleep(5);
 		cprintf("%d Authentication failed.\n", ERROR + PASSWORD_REQUIRED);
-		CtdlLogPrintf(CTDL_ERR, "Warning: ipgm authentication failed.\n");
-		CC->kill_me = 1;
+		syslog(LOG_ERR, "Warning: ipgm authentication failed.\n");
+		CC->kill_me = KILLME_AUTHFAILED;
 	}
 }
 
@@ -794,7 +783,7 @@ void cmd_down(char *argbuf) {
 		}
 		if ((restart_server > 0) && !running_as_daemon)
 		{
-			CtdlLogPrintf(CTDL_ERR, "The user requested restart, but not running as daemon! Geronimooooooo!\n");
+			syslog(LOG_ERR, "The user requested restart, but not running as daemon! Geronimooooooo!\n");
 			Reply = "%d Warning: citserver is not running in daemon mode and is therefore unlikely to restart automatically.\n";
 			state = ERROR;
 		}
@@ -804,8 +793,8 @@ void cmd_down(char *argbuf) {
 	{
 		cprintf(Reply, CIT_OK + SERVER_SHUTTING_DOWN); 
 	}
-	CC->kill_me = 1; /* Even the DOWN command has to follow correct proceedure when disconecting */
-	CtdlThreadStopAll();
+	CC->kill_me = KILLME_SERVER_SHUTTING_DOWN;
+	server_shutting_down = 1;
 }
 
 
@@ -817,7 +806,7 @@ void cmd_halt(char *argbuf) {
 	if (CtdlAccessCheck(ac_aide)) return;
 
 	cprintf("%d Halting server.  Goodbye.\n", CIT_OK);
-	CtdlThreadStopAll();
+	server_shutting_down = 1;
 	shutdown_and_halt = 1;
 }
 
@@ -839,7 +828,7 @@ void cmd_scdn(char *argbuf)
 		restart_server = 1;
 		if (!running_as_daemon)
 		{
-			CtdlLogPrintf(CTDL_ERR, "The user requested restart, but not running as deamon! Geronimooooooo!\n");
+			syslog(LOG_ERR, "The user requested restart, but not running as deamon! Geronimooooooo!\n");
 			Reply = "%d %d Warning, not running in deamon mode. maybe we will come up again, but don't lean on it.\n";
 			state = ERROR;
 		}
@@ -869,31 +858,12 @@ void cmd_asyn(char *argbuf)
 }
 
 
-/*
- * Generate a "nonce" for APOP-style authentication.
- *
- * RFC 1725 et al specify a PID to be placed in front of the nonce.
- * Quoth BTX: That would be stupid.
- */
-void generate_nonce(CitContext *con) {
-	struct timeval tv;
-
-	memset(con->cs_nonce, NONCE_SIZE, 0);
-	gettimeofday(&tv, NULL);
-	memset(con->cs_nonce, NONCE_SIZE, 0);
-	snprintf(con->cs_nonce, NONCE_SIZE, "<%d%ld@%s>",
-		rand(), (long)tv.tv_usec, config.c_fqdn);
-}
-
 
 /*
  * Back-end function for starting a session
  */
 void begin_session(CitContext *con)
 {
-	socklen_t len;
-	struct sockaddr_in sin;
-
 	/* 
 	 * Initialize some variables specific to our context.
 	 */
@@ -901,6 +871,8 @@ void begin_session(CitContext *con)
 	con->internal_pgm = 0;
 	con->download_fp = NULL;
 	con->upload_fp = NULL;
+	con->cached_msglist = NULL;
+	con->cached_num_msgs = 0;
 	con->FirstExpressMessage = NULL;
 	time(&con->lastcmd);
 	time(&con->lastidle);
@@ -911,22 +883,20 @@ void begin_session(CitContext *con)
 	*con->fake_username = '\0';
 	*con->fake_hostname = '\0';
 	*con->fake_roomname = '\0';
-	generate_nonce(con);
+	*con->cs_clientinfo = '\0';
 	safestrncpy(con->cs_host, config.c_fqdn, sizeof con->cs_host);
 	safestrncpy(con->cs_addr, "", sizeof con->cs_addr);
 	con->cs_UDSclientUID = -1;
 	con->cs_host[sizeof con->cs_host - 1] = 0;
-	len = sizeof sin;
 	if (!CC->is_local_socket) {
-		if (!getpeername(con->client_socket, (struct sockaddr *) &sin, &len)) {
-			locate_host(con->cs_host, sizeof con->cs_host,
-				con->cs_addr, sizeof con->cs_addr,
-				&sin.sin_addr
-			);
-		}
+		locate_host(con->cs_host, sizeof con->cs_host,
+			con->cs_addr, sizeof con->cs_addr,
+			con->client_socket
+		);
 	}
 	else {
-		strcpy(con->cs_host, "");
+		con->cs_host[0] = 0;
+		con->cs_addr[0] = 0;
 #ifdef HAVE_STRUCT_UCRED
 		{
 			/* as http://www.wsinnovations.com/softeng/articles/uds.html told us... */
@@ -935,7 +905,7 @@ void begin_session(CitContext *con)
 			
 			/*fill in the user data structure */
 			if(getsockopt(con->client_socket, SOL_SOCKET, SO_PEERCRED, &credentials, &ucred_length)) {
-				CtdlLogPrintf(CTDL_NOTICE, "could obtain credentials from unix domain socket");
+				syslog(LOG_NOTICE, "could obtain credentials from unix domain socket");
 				
 			}
 			else {		
@@ -952,6 +922,11 @@ void begin_session(CitContext *con)
 				   database, after a reverse lookup on the UID to get the account name.
 				   We can take this opportunity to check to see if this is a legit account.
 				*/
+				snprintf(con->cs_clientinfo, sizeof(con->cs_clientinfo),
+					 "PID: "F_PID_T"; UID: "F_UID_T"; GID: "F_XPID_T" ", 
+					 credentials.pid,
+					 credentials.uid,
+					 credentials.gid);
 			}
 		}
 #endif
@@ -966,10 +941,10 @@ void begin_session(CitContext *con)
 	}
 
 	if (!CC->is_local_socket) {
-		CtdlLogPrintf(CTDL_NOTICE, "Session (%s) started from %s (%s).\n", con->ServiceName, con->cs_host, con->cs_addr);
+		syslog(LOG_NOTICE, "Session (%s) started from %s (%s).\n", con->ServiceName, con->cs_host, con->cs_addr);
 	}
 	else {
-		CtdlLogPrintf(CTDL_NOTICE, "Session (%s) started via local socket UID:%d.\n", con->ServiceName, con->cs_UDSclientUID);
+		syslog(LOG_NOTICE, "Session (%s) started via local socket UID:%d.\n", con->ServiceName, con->cs_UDSclientUID);
 	}
 
 	/* Run any session startup routines registered by loadable modules */
@@ -983,7 +958,7 @@ void citproto_begin_session() {
 			ERROR + MAX_SESSIONS_EXCEEDED,
 			config.c_nodename, config.c_maxsessions
 		);
-		CC->kill_me = 1;
+		CC->kill_me = KILLME_MAX_SESSIONS_EXCEEDED;
 	}
 	else {
 		cprintf("%d %s Citadel server ready.\n", CIT_OK, config.c_nodename);
@@ -1007,7 +982,7 @@ void cmd_qnop(char *argbuf)
 void cmd_quit(char *argbuf)
 {
 	cprintf("%d Goodbye.\n", CIT_OK);
-	CC->kill_me = 1;
+	CC->kill_me = KILLME_CLIENT_LOGGED_OUT;
 }
 
 
@@ -1024,25 +999,25 @@ void cmd_lout(char *argbuf)
  */
 void do_command_loop(void) {
 	char cmdbuf[SIZ];
-	const char *old_name = NULL;
-	
-	old_name = CtdlThreadName("do_command_loop");
 	
 	time(&CC->lastcmd);
 	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
 	if (client_getln(cmdbuf, sizeof cmdbuf) < 1) {
-		CtdlLogPrintf(CTDL_ERR, "Client disconnected: ending session.\n");
-		CC->kill_me = 1;
-		CtdlThreadName(old_name);
+		syslog(LOG_ERR, "Citadel client disconnected: ending session.\n");
+		CC->kill_me = KILLME_CLIENT_DISCONNECTED;
 		return;
 	}
 
 	/* Log the server command, but don't show passwords... */
 	if ( (strncasecmp(cmdbuf, "PASS", 4)) && (strncasecmp(cmdbuf, "SETP", 4)) ) {
-		CtdlLogPrintf(CTDL_INFO, "%s\n", cmdbuf);
+		syslog(LOG_INFO, "[%d][%s(%ld)] %s",
+			CC->cs_pid, CC->curr_user, CC->user.usernum, cmdbuf
+		);
 	}
 	else {
-		CtdlLogPrintf(CTDL_INFO, "<password command hidden from log>\n");
+		syslog(LOG_INFO, "[%d][%s(%ld)] <password command hidden from log>",
+			CC->cs_pid, CC->curr_user, CC->user.usernum
+		);
 	}
 
 	buffer_output();
@@ -1062,8 +1037,6 @@ void do_command_loop(void) {
 		time(&CC->lastidle);
 	}
 	
-	CtdlThreadName(cmdbuf);
-		
 	if ((strncasecmp(cmdbuf, "ENT0", 4))
 	   && (strncasecmp(cmdbuf, "MESG", 4))
 	   && (strncasecmp(cmdbuf, "MSGS", 4)))
@@ -1079,7 +1052,6 @@ void do_command_loop(void) {
 
 	/* Run any after-each-command routines registered by modules */
 	PerformSessionHooks(EVT_CMD);
-	CtdlThreadName(old_name);
 }
 
 
@@ -1116,6 +1088,6 @@ CTDL_MODULE_INIT(citserver)
 		CtdlRegisterProtoHook(cmd_scdn, "SCDN", "schedule or cancel a server shutdown");
 		CtdlRegisterProtoHook(cmd_time, "TIME", "fetch the date and time from the server");
 	}
-        /* return our Subversion id for the Log */
-	return "$Id$";
+        /* return our id for the Log */
+	return "citserver";
 }
