@@ -1,5 +1,4 @@
-/* $Id$
- *
+/*
  * Copyright (c) 1987-2009 by the citadel.org team
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -30,6 +29,8 @@ struct UserProcList {
 	char user[64];
 };
 
+#define CTDLUSERIP      (IsEmptyStr(CC->cs_addr) ?  CC->cs_clientinfo: CC->cs_addr)
+
 void cit_backtrace(void);
 void cit_panic_backtrace(int SigNum);
 void master_startup (void);
@@ -45,10 +46,5 @@ void GenerateRoomDisplay(char *real_room,
                         CitContext *viewer);
 extern int panic_fd;
 char CtdlCheckExpress(void);
-
-
-
-
-
 extern time_t server_startup_time;
-
+extern int openid_level_supported;
