@@ -4,21 +4,21 @@
  * sockets for the Citadel client; for that you must look in ipc_c_tcp.c
  * (which, uncoincidentally, bears a striking similarity to this file).
  *
- * Copyright (c) 1987-2009 by the citadel.org team
+ * Copyright (c) 1987-2011 by the citadel.org team
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is open source software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as published
+ * by the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "sysdep.h"
@@ -36,6 +36,7 @@
 #include <pwd.h>
 #include <errno.h>
 #include <stdarg.h>
+#include <syslog.h>
 #include <libcitadel.h>
 #include "citadel.h"
 #include "server.h"
@@ -45,89 +46,72 @@
 #include "sysdep_decls.h"
 #include "config.h"
 #include "clientsocket.h"
-
 #include "ctdl_module.h"
 
-#ifndef INADDR_NONE
-#define INADDR_NONE 0xffffffff
-#endif
-
-int sock_connect(char *host, char *service, char *protocol)
+int sock_connect(char *host, char *service)
 {
-	struct hostent *phe;
-	struct servent *pse;
-	struct protoent *ppe;
-	struct sockaddr_in sin;
-	struct sockaddr_in egress_sin;
-	int s, type;
+	struct in6_addr serveraddr;
+	struct addrinfo hints;
+	struct addrinfo *res = NULL;
+	struct addrinfo *ai = NULL;
+	int rc = (-1);
+	int sock = (-1);
 
-	if ((host == NULL) || IsEmptyStr(host)) 
-		return(-1);
-	if ((service == NULL) || IsEmptyStr(service)) 
-		return(-1);
-	if ((protocol == NULL) || IsEmptyStr(protocol)) 
-		return(-1);
+	if ((host == NULL) || IsEmptyStr(host))
+		return (-1);
+	if ((service == NULL) || IsEmptyStr(service))
+		return (-1);
 
-	memset(&sin, 0, sizeof(sin));
-	sin.sin_family = AF_INET;
+	memset(&hints, 0x00, sizeof(hints));
+	hints.ai_flags = AI_NUMERICSERV;
+	hints.ai_family = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
 
-	pse = getservbyname(service, protocol);
-	if (pse) {
-		sin.sin_port = pse->s_port;
-	} else if ((sin.sin_port = htons((u_short) atoi(service))) == 0) {
-		CtdlLogPrintf(CTDL_CRIT, "Can't get %s service entry: %s\n",
-			service, strerror(errno));
-		return(-1);
-	}
-	phe = gethostbyname(host);
-	if (phe) {
-		memcpy(&sin.sin_addr, phe->h_addr, phe->h_length);
-	} else if ((sin.sin_addr.s_addr = inet_addr(host)) == INADDR_NONE) {
-		CtdlLogPrintf(CTDL_ERR, "Can't get %s host entry: %s\n",
-			host, strerror(errno));
-		return(-1);
-	}
-	if ((ppe = getprotobyname(protocol)) == 0) {
-		CtdlLogPrintf(CTDL_CRIT, "Can't get %s protocol entry: %s\n",
-			protocol, strerror(errno));
-		return(-1);
-	}
-	if (!strcmp(protocol, "udp")) {
-		type = SOCK_DGRAM;
-	} else {
-		type = SOCK_STREAM;
-	}
-
-	s = socket(PF_INET, type, ppe->p_proto);
-	if (s < 0) {
-		CtdlLogPrintf(CTDL_CRIT, "Can't create socket: %s\n", strerror(errno));
-		return(-1);
-	}
-
-	/* If citserver is bound to a specific IP address on the host, make
-	 * sure we use that address for outbound connections.
+	/*
+	 * Handle numeric IPv4 and IPv6 addresses
 	 */
-	memset(&egress_sin, 0, sizeof(egress_sin));
-	egress_sin.sin_family = AF_INET;
-	if (!IsEmptyStr(config.c_ip_addr)) {
-		egress_sin.sin_addr.s_addr = inet_addr(config.c_ip_addr);
-        	if (egress_sin.sin_addr.s_addr == !INADDR_ANY) {
-                	egress_sin.sin_addr.s_addr = INADDR_ANY;
+	rc = inet_pton(AF_INET, host, &serveraddr);
+	if (rc == 1) {						/* dotted quad */
+		hints.ai_family = AF_INET;
+		hints.ai_flags |= AI_NUMERICHOST;
+	} else {
+		rc = inet_pton(AF_INET6, host, &serveraddr);
+		if (rc == 1) {					/* IPv6 address */
+			hints.ai_family = AF_INET6;
+			hints.ai_flags |= AI_NUMERICHOST;
 		}
+	}
 
-		/* If this bind fails, no problem; we can still use INADDR_ANY */
-		bind(s, (struct sockaddr *)&egress_sin, sizeof(egress_sin));
-        }
+	/* Begin the connection process */
 
-	/* Now try to connect to the remote host. */
-	if (connect(s, (struct sockaddr *) &sin, sizeof(sin)) < 0) {
-		CtdlLogPrintf(CTDL_ERR, "Can't connect to %s:%s: %s\n",
-			host, service, strerror(errno));
-		close(s);
+	rc = getaddrinfo(host, service, &hints, &res);
+	if (rc != 0) {
+		syslog(LOG_ERR, "%s: %s", host, gai_strerror(rc));
 		return(-1);
 	}
 
-	return (s);
+	/*
+	 * Try all available addresses until we connect to one or until we run out.
+	 */
+	for (ai = res; ai != NULL; ai = ai->ai_next) {
+		sock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
+		if (sock < 0) {
+			syslog(LOG_ERR, "socket() failed: %s", strerror(errno));
+			freeaddrinfo(res);
+			return(-1);
+		}
+		rc = connect(sock, ai->ai_addr, ai->ai_addrlen);
+		if (rc >= 0) {
+                        freeaddrinfo(res);
+			return(sock);
+		}
+		else {
+			syslog(LOG_ERR, "connect() failed: %s", strerror(errno));
+			close(sock);
+		}
+	}
+	freeaddrinfo(res);
+	return(-1);
 }
 
 
@@ -145,79 +129,38 @@ int sock_connect(char *host, char *service, char *protocol)
  *      0       Request timed out.
  *	-1   	Connection is broken, or other error.
  */
-int socket_read_blob(int *Socket, 
-		     StrBuf *Target, 
-		     int bytes, 
-		     int timeout)
+int socket_read_blob(int *Socket, StrBuf * Target, int bytes, int timeout)
 {
-	CitContext *CCC=CC;
+	CitContext *CCC = MyContext();
 	const char *Error;
 	int retval = 0;
 
 
-	retval = StrBufReadBLOBBuffered(Target, 
+	retval = StrBufReadBLOBBuffered(Target,
 					CCC->sReadBuf,
 					&CCC->sPos,
-					Socket,
-					1, 
-					bytes,
-					O_TERM,
-					&Error);
+					Socket, 1, bytes, O_TERM, &Error
+	);
 	if (retval < 0) {
-		CtdlLogPrintf(CTDL_CRIT, 
-			      "%s failed: %s\n",
-			      __FUNCTION__,
-			      Error);
+		syslog(LOG_CRIT, "socket_read_blob() failed: %s", Error);
 	}
 	return retval;
 }
 
 
-int sock_read_to(int *sock, char *buf, int bytes, int timeout, int keep_reading_until_full)
-{
-	CitContext *CCC=CC;
-	int rc;
-
-	FlushStrBuf(CCC->MigrateBuf);
-	rc = socket_read_blob(sock, 
-			      CCC->sMigrateBuf, 
-			      bytes, 
-			      timeout);
-	if (rc < 0)
-	{
-		*buf = '\0';
-		return rc;
-	}
-	else
-	{
-		if (StrLength(CCC->MigrateBuf) < bytes)
-			bytes = StrLength(CCC->MigrateBuf);
-		memcpy(buf, 
-		       ChrPtr(CCC->MigrateBuf),
-		       bytes);
-
-		FlushStrBuf(CCC->MigrateBuf);
-		return rc;
-	}
-}
-
-
 int CtdlSockGetLine(int *sock, StrBuf * Target, int nSec)
 {
-	CitContext *CCC=CC;
+	CitContext *CCC = MyContext();
 	const char *Error;
 	int rc;
 
 	FlushStrBuf(Target);
-	rc = StrBufTCP_read_buffered_line_fast(Target, 
+	rc = StrBufTCP_read_buffered_line_fast(Target,
 					       CCC->sReadBuf,
 					       &CCC->sPos,
 					       sock, nSec, 1, &Error);
 	if ((rc < 0) && (Error != NULL))
-		CtdlLogPrintf(CTDL_CRIT, 
-			      "%s failed: %s\n",
-			      __FUNCTION__,
-			      Error);
+		syslog(LOG_CRIT, "CtdlSockGetLine() failed: %s", Error);
 	return rc;
 }
 
@@ -230,7 +173,7 @@ int CtdlSockGetLine(int *sock, StrBuf * Target, int nSec)
 int sock_getln(int *sock, char *buf, int bufsize)
 {
 	int i, retval;
-	CitContext *CCC=CC;
+	CitContext *CCC = MyContext();
 	const char *pCh;
 
 	FlushStrBuf(CCC->sMigrateBuf);
@@ -250,16 +193,6 @@ int sock_getln(int *sock, char *buf, int bufsize)
 }
 
 
-/*
- * sock_read() - input binary data from socket.
- * Returns the number of bytes read, or -1 for error.
- */
-INLINE int sock_read(int *sock, char *buf, int bytes, int keep_reading_until_full)
-{
-	return sock_read_to(sock, buf, bytes, CLIENT_TIMEOUT, keep_reading_until_full);
-}
-
-
 /*
  * sock_write() - send binary to server.
  * Returns the number of bytes written, or -1 for error.
@@ -360,7 +293,6 @@ int sock_getln_err(int *sock, char *buf, int bufsize, int *rc, int nSec)
  * client side protocol implementations.  It only returns the first line of
  * a multiline response, discarding the rest.
  */
-
 int ml_sock_gets(int *sock, char *buf, int nSec)
 {
 	int rc = 0;
@@ -383,7 +315,7 @@ int ml_sock_gets(int *sock, char *buf, int nSec)
 			return (g);
 	} while ((g >= 4) && (bigbuf[3] == '-'));
 
-	return(strlen(buf));
+	return (strlen(buf));
 }
 
 
@@ -396,8 +328,10 @@ int sock_puts(int *sock, char *buf)
 	int i, j;
 
 	i = sock_write(sock, buf, strlen(buf));
-	if (i<0) return(i);
+	if (i < 0)
+		return (i);
 	j = sock_write(sock, "\n", 1);
-	if (j<0) return(j);
-	return(i+j);
+	if (j < 0)
+		return (j);
+	return (i + j);
 }
