@@ -1,5 +1,21 @@
 /*
  * This file contains miscellaneous housekeeping tasks.
+ *
+ * Copyright (c) 1987-2011 by the citadel.org team
+ *
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "sysdep.h"
@@ -27,6 +43,7 @@
 #ifdef HAVE_SYS_SELECT_H
 #include <sys/select.h>
 #endif
+#include <syslog.h>
 #include <libcitadel.h>
 #include "citadel.h"
 #include "server.h"
@@ -45,8 +62,8 @@
 
 void check_sched_shutdown(void) {
 	if ((ScheduledShutdown == 1) && (ContextList == NULL)) {
-		CtdlLogPrintf(CTDL_NOTICE, "Scheduled shutdown initiating.\n");
-		CtdlThreadStopAll();
+		syslog(LOG_NOTICE, "Scheduled shutdown initiating.\n");
+		server_shutting_down = 1;
 	}
 }
 
@@ -71,7 +88,7 @@ void check_ref_counts(void) {
 
 	int new_refcounts[MAXFLOORS];
 
-	CtdlLogPrintf(CTDL_DEBUG, "Checking floor reference counts\n");
+	syslog(LOG_DEBUG, "Checking floor reference counts\n");
 	for (a=0; a<MAXFLOORS; ++a) {
 		new_refcounts[a] = 0;
 	}
@@ -90,7 +107,7 @@ void check_ref_counts(void) {
 			flbuf.f_flags = flbuf.f_flags & ~QR_INUSE;
 		}
 		lputfloor(&flbuf, a);
-		CtdlLogPrintf(CTDL_DEBUG, "Floor %d: %d rooms\n", a, new_refcounts[a]);
+		syslog(LOG_DEBUG, "Floor %d: %d rooms\n", a, new_refcounts[a]);
 	}
 }	
 
@@ -106,7 +123,6 @@ void do_housekeeping(void) {
 	int do_housekeeping_now = 0;
 	int do_perminute_housekeeping_now = 0;
 	time_t now;
-	const char *old_name;
 
 	/*
 	 * We do it this way instead of wrapping the whole loop in an
@@ -135,16 +151,12 @@ void do_housekeeping(void) {
 	 */
 
 	/* First, do the "as often as needed" stuff... */
-	old_name = CtdlThreadName("House Keeping - Journal");
 	JournalRunQueue();
-
-	CtdlThreadName("House Keeping - EVT_HOUSE");
-	PerformSessionHooks(EVT_HOUSE);	/* perform as needed housekeeping */
+	PerformSessionHooks(EVT_HOUSE);
 
 	/* Then, do the "once per minute" stuff... */
 	if (do_perminute_housekeeping_now) {
 		cdb_check_handles();			/* suggested by Justin Case */
-		CtdlThreadName("House Keeping - EVT_TIMER");
 		PerformSessionHooks(EVT_TIMER);		/* Run any timer hooks */
 	}
 
@@ -152,5 +164,4 @@ void do_housekeeping(void) {
 	 * All done.
 	 */
 	housekeeping_in_progress = 0;
-	CtdlThreadName(old_name);
 }
