@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * This module allows Citadel to use clamd to filter incoming messages
  * arriving via SMTP.  For more information on clamd, visit
  * http://clamav.net (the ClamAV project is not in any way
@@ -96,18 +94,18 @@ int clamd(struct CtdlMessage *msg) {
 	/* Try them one by one until we get a working one */
         for (clamhost=0; clamhost<num_clamhosts; ++clamhost) {
                 extract_token(buf, clamhosts, clamhost, '|', sizeof buf);
-                CtdlLogPrintf(CTDL_INFO, "Connecting to clamd at <%s>\n", buf);
+                syslog(LOG_INFO, "Connecting to clamd at <%s>\n", buf);
 
                 /* Assuming a host:port entry */ 
                 extract_token(hostbuf, buf, 0, ':', sizeof hostbuf);
                 if (extract_token(portbuf, buf, 1, ':', sizeof portbuf)==-1)
                   /* Didn't specify a port so we'll try the psuedo-standard 3310 */
-                  sock = sock_connect(hostbuf, CLAMD_PORT, "tcp");
+                  sock = sock_connect(hostbuf, CLAMD_PORT);
                 else
                   /* Port specified lets try connecting to it! */
-                  sock = sock_connect(hostbuf, portbuf, "tcp");
+                  sock = sock_connect(hostbuf, portbuf);
 
-                if (sock >= 0) CtdlLogPrintf(CTDL_DEBUG, "Connected!\n");
+                if (sock >= 0) syslog(LOG_DEBUG, "Connected!\n");
         }
 
 	if (sock < 0) {
@@ -122,16 +120,16 @@ int clamd(struct CtdlMessage *msg) {
 	CCC->sPos = NULL;
 
 	/* Command */
-	CtdlLogPrintf(CTDL_DEBUG, "Transmitting STREAM command\n");
+	syslog(LOG_DEBUG, "Transmitting STREAM command\n");
 	sprintf(buf, "STREAM\r\n");
 	sock_write(&sock, buf, strlen(buf));
 
-	CtdlLogPrintf(CTDL_DEBUG, "Waiting for PORT number\n");
+	syslog(LOG_DEBUG, "Waiting for PORT number\n");
         if (sock_getln(&sock, buf, sizeof buf) < 0) {
                 goto bail;
         }
 
-        CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
+        syslog(LOG_DEBUG, "<%s\n", buf);
 	if (strncasecmp(buf, "PORT", 4)!=0) {
 	        goto bail;
 	}
@@ -140,7 +138,7 @@ int clamd(struct CtdlMessage *msg) {
 	extract_token(portbuf, buf, 1, ' ', sizeof portbuf);
 
 	/* Attempt to establish connection to STREAM socket */
-        streamsock = sock_connect(hostbuf, portbuf, "tcp");
+        streamsock = sock_connect(hostbuf, portbuf);
 
 	if (streamsock < 0) {
 		/* If the service isn't running, just pass the mail
@@ -151,7 +149,7 @@ int clamd(struct CtdlMessage *msg) {
 		return(0);
         }
 	else {
-	        CtdlLogPrintf(CTDL_DEBUG, "STREAM socket connected!\n");
+	        syslog(LOG_DEBUG, "STREAM socket connected!\n");
 	}
 
 
@@ -172,11 +170,11 @@ int clamd(struct CtdlMessage *msg) {
 		close(streamsock);
 	
 	/* Response */
-	CtdlLogPrintf(CTDL_DEBUG, "Awaiting response\n");
+	syslog(LOG_DEBUG, "Awaiting response\n");
         if (sock_getln(&sock, buf, sizeof buf) < 0) {
                 goto bail;
         }
-        CtdlLogPrintf(CTDL_DEBUG, "<%s\n", buf);
+        syslog(LOG_DEBUG, "<%s\n", buf);
 	if (strncasecmp(buf, "stream: OK", 10)!=0) {
 		is_virus = 1;
 	}
@@ -203,6 +201,6 @@ CTDL_MODULE_INIT(virus)
 		CtdlRegisterMessageHook(clamd, EVT_SMTPSCAN);
 	}
 	
-	/* return our Subversion id for the Log */
-        return "$Id$";
+	/* return our module name for the log */
+        return "virus";
 }
