@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * This module glues libDSpam to the Citadel server in order to implement
  * DSPAM Spamchecking 
  *
@@ -159,12 +157,12 @@ void dspam_do_msg(long msgnum, void *userdata)
 	if (dspam_process (CTX, msgtext) != 0)
 	{
 		free(msgtext);
-		CtdlLogPrintf(CTDL_CRIT, "ERROR: dspam_process failed");
+		syslog(LOG_CRIT, "ERROR: dspam_process failed");
 		return;
 	}
 	if (CTX->signature == NULL)
 	{
-		CtdlLogPrintf(CTDL_CRIT,"No signature provided\n");
+		syslog(LOG_CRIT,"No signature provided\n");
 	}
 	else
 	{
@@ -177,7 +175,7 @@ void dspam_do_msg(long msgnum, void *userdata)
 
 	SIG.length = CTX->signature->length;
 	/* Print processing results */
-	CtdlLogPrintf (CTDL_DEBUG, "Probability: %2.4f Confidence: %2.4f, Result: %s\n",
+	syslog(LOG_DEBUG, "Probability: %2.4f Confidence: %2.4f, Result: %s\n",
 		CTX->probability,
 		CTX->confidence,
 		(CTX->result == DSR_ISSPAM) ? "Spam" : "Innocent");
@@ -216,7 +214,7 @@ int serv_dspam_room(struct ctdlroom *room)
 	/// else -> todo: global user for public rooms etc.
 	if (CTX == NULL)
 	{
-		CtdlLogPrintf(CTDL_CRIT, "ERROR: dspam_init failed!\n");
+		syslog(LOG_CRIT, "ERROR: dspam_init failed!\n");
 		return ERROR + INTERNAL_ERROR;
 	}
 	/* Use graham and robinson algorithms, graham's p-values */
@@ -240,7 +238,7 @@ void serv_dspam_shutdown (void)
 
 CTDL_MODULE_INIT(dspam)
 {
-	return "$Id$" "disabled.";
+	return "disabled.";
 	if (!threading)
 	{
 #ifdef HAVE_LIBDSPAM
@@ -255,12 +253,12 @@ CTDL_MODULE_INIT(dspam)
 
 #else	/* HAVE_LIBDSPAM */
 
-		CtdlLogPrintf(CTDL_INFO, "This server is missing libdspam Spam filtering will be disabled.\n");
+		syslog(LOG_INFO, "This server is missing libdspam Spam filtering will be disabled.\n");
 
 #endif	/* HAVE_LIBDSPAM */
 	}
 	
-        /* return our Subversion id for the Log */
-	return "$Id$";
+        /* return our module name for the log */
+	return "dspam";
 }
 
