@@ -1,6 +1,4 @@
 /* 
- * $Id$
- *
  * Functions which manage policy for rooms (such as message expiry)
  */
 
@@ -85,7 +83,8 @@ void cmd_gpex(char *argbuf) {
 	char which[128];
 
 	extract_token(which, argbuf, 0, '|', sizeof which);
-	if (!strcasecmp(which, strof(room))) {
+	if (!strcasecmp(which, strof(roompolicy))||
+	    !strcasecmp(which, "room")) { /* Deprecated version */
 		memcpy(&exp, &CC->room.QRep, sizeof(struct ExpirePolicy));
 	}
 	else if (!strcasecmp(which, strof(floorpolicy))||
@@ -128,7 +127,8 @@ void cmd_spex(char *argbuf) {
 		return;
 	}
 
-	if (!strcasecmp(which, "room")) {
+	if (!strcasecmp(which, strof(roompolicy))||
+	    !strcasecmp(which, "room")) {
 		if (!is_room_aide()) {
 			cprintf("%d Higher access required.\n",
 				ERROR + HIGHER_ACCESS_REQUIRED);
