@@ -1,10 +1,7 @@
 /*
- * $Id$
- *
  * Implements IMAP's gratuitously complex SEARCH command.
  *
- *
- * Copyright (c) 2001-2009 by the citadel.org team
+ * Copyright (c) 2001-2011 by the citadel.org team
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -60,8 +57,8 @@
 #include "database.h"
 #include "msgbase.h"
 #include "internet_addressing.h"
-#include "imap_tools.h"
 #include "serv_imap.h"
+#include "imap_tools.h"
 #include "imap_fetch.h"
 #include "imap_search.h"
 #include "genstamp.h"
@@ -79,6 +76,7 @@
 int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 			int num_items, ConstStr *itemlist, int is_uid) {
 
+	citimap *Imap = IMAP;
 	int match = 0;
 	int is_not = 0;
 	int is_or = 0;
@@ -115,7 +113,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 	}
 	
 	else if (!strcasecmp(itemlist[pos].Key, "ANSWERED")) {
-		if (IMAP->flags[seq-1] & IMAP_ANSWERED) {
+		if (Imap->flags[seq-1] & IMAP_ANSWERED) {
 			match = 1;
 		}
 		++pos;
@@ -123,7 +121,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "BCC")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -140,7 +138,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "BEFORE")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -166,7 +164,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 		/* Otherwise, we have to do a slow search. */
 		else {
 			if (msg == NULL) {
-				msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+				msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 				need_to_free_msg = 1;
 			}
 			if (msg != NULL) {
@@ -181,7 +179,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "CC")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -205,21 +203,21 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "DELETED")) {
-		if (IMAP->flags[seq-1] & IMAP_DELETED) {
+		if (Imap->flags[seq-1] & IMAP_DELETED) {
 			match = 1;
 		}
 		++pos;
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "DRAFT")) {
-		if (IMAP->flags[seq-1] & IMAP_DRAFT) {
+		if (Imap->flags[seq-1] & IMAP_DRAFT) {
 			match = 1;
 		}
 		++pos;
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "FLAGGED")) {
-		if (IMAP->flags[seq-1] & IMAP_FLAGGED) {
+		if (Imap->flags[seq-1] & IMAP_FLAGGED) {
 			match = 1;
 		}
 		++pos;
@@ -227,7 +225,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "FROM")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -249,7 +247,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 		 * examining the message body.
 		 */
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 
@@ -279,7 +277,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "LARGER")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -291,14 +289,14 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "NEW")) {
-		if ( (IMAP->flags[seq-1] & IMAP_RECENT) && (!(IMAP->flags[seq-1] & IMAP_SEEN))) {
+		if ( (Imap->flags[seq-1] & IMAP_RECENT) && (!(Imap->flags[seq-1] & IMAP_SEEN))) {
 			match = 1;
 		}
 		++pos;
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "OLD")) {
-		if (!(IMAP->flags[seq-1] & IMAP_RECENT)) {
+		if (!(Imap->flags[seq-1] & IMAP_RECENT)) {
 			match = 1;
 		}
 		++pos;
@@ -306,7 +304,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "ON")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -321,14 +319,14 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "RECENT")) {
-		if (IMAP->flags[seq-1] & IMAP_RECENT) {
+		if (Imap->flags[seq-1] & IMAP_RECENT) {
 			match = 1;
 		}
 		++pos;
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "SEEN")) {
-		if (IMAP->flags[seq-1] & IMAP_SEEN) {
+		if (Imap->flags[seq-1] & IMAP_SEEN) {
 			match = 1;
 		}
 		++pos;
@@ -336,7 +334,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "SENTBEFORE")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -352,7 +350,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "SENTON")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -368,7 +366,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "SENTSINCE")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -384,7 +382,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "SINCE")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -400,7 +398,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "SMALLER")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -413,7 +411,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "SUBJECT")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -426,7 +424,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "TEXT")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -441,7 +439,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	else if (!strcasecmp(itemlist[pos].Key, "TO")) {
 		if (msg == NULL) {
-			msg = CtdlFetchMessage(IMAP->msgids[seq-1], 1);
+			msg = CtdlFetchMessage(Imap->msgids[seq-1], 1);
 			need_to_free_msg = 1;
 		}
 		if (msg != NULL) {
@@ -462,7 +460,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 
 	/* FIXME this is b0rken.  fix it. */
 	else if (!strcasecmp(itemlist[pos].Key, "UID")) {
-		if (is_msg_in_sequence_set(itemlist[pos+1].Key, IMAP->msgids[seq-1])) {
+		if (is_msg_in_sequence_set(itemlist[pos+1].Key, Imap->msgids[seq-1])) {
 			match = 1;
 		}
 		pos += 2;
@@ -474,28 +472,28 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 	 */
 
 	else if (!strcasecmp(itemlist[pos].Key, "UNANSWERED")) {
-		if ((IMAP->flags[seq-1] & IMAP_ANSWERED) == 0) {
+		if ((Imap->flags[seq-1] & IMAP_ANSWERED) == 0) {
 			match = 1;
 		}
 		++pos;
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "UNDELETED")) {
-		if ((IMAP->flags[seq-1] & IMAP_DELETED) == 0) {
+		if ((Imap->flags[seq-1] & IMAP_DELETED) == 0) {
 			match = 1;
 		}
 		++pos;
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "UNDRAFT")) {
-		if ((IMAP->flags[seq-1] & IMAP_DRAFT) == 0) {
+		if ((Imap->flags[seq-1] & IMAP_DRAFT) == 0) {
 			match = 1;
 		}
 		++pos;
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "UNFLAGGED")) {
-		if ((IMAP->flags[seq-1] & IMAP_FLAGGED) == 0) {
+		if ((Imap->flags[seq-1] & IMAP_FLAGGED) == 0) {
 			match = 1;
 		}
 		++pos;
@@ -507,7 +505,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
 	}
 
 	else if (!strcasecmp(itemlist[pos].Key, "UNSEEN")) {
-		if ((IMAP->flags[seq-1] & IMAP_SEEN) == 0) {
+		if ((Imap->flags[seq-1] & IMAP_SEEN) == 0) {
 			match = 1;
 		}
 		++pos;
@@ -544,6 +542,7 @@ int imap_do_search_msg(int seq, struct CtdlMessage *supplied_msg,
  * validated and boiled down the request a bit.
  */
 void imap_do_search(int num_items, ConstStr *itemlist, int is_uid) {
+	citimap *Imap = IMAP;
 	int i, j, k;
 	int fts_num_msgs = 0;
 	long *fts_msgs = NULL;
@@ -557,12 +556,12 @@ void imap_do_search(int num_items, ConstStr *itemlist, int is_uid) {
 	for (i=0; i<num_items; ++i) {
 		if (itemlist[i].Key[0] == '(') {
 			
-			TokenCutLeft(&IMAP->Cmd, 
+			TokenCutLeft(&Imap->Cmd, 
 				     &itemlist[i], 
 				     1);
 		}
 		if (itemlist[i].Key[itemlist[i].len-1] == ')') {
-			TokenCutRight(&IMAP->Cmd, 
+			TokenCutRight(&Imap->Cmd, 
 				      &itemlist[i], 
 				      1);
 		}
@@ -576,23 +575,23 @@ void imap_do_search(int num_items, ConstStr *itemlist, int is_uid) {
 		if (!strcasecmp(itemlist[i].Key, "BODY")) {
 			CtdlModuleDoSearch(&fts_num_msgs, &fts_msgs, itemlist[i+1].Key, "fulltext");
 			if (fts_num_msgs > 0) {
-				for (j=0; j < IMAP->num_msgs; ++j) {
-					if (IMAP->flags[j] & IMAP_SELECTED) {
+				for (j=0; j < Imap->num_msgs; ++j) {
+					if (Imap->flags[j] & IMAP_SELECTED) {
 						is_in_list = 0;
 						for (k=0; k<fts_num_msgs; ++k) {
-							if (IMAP->msgids[j] == fts_msgs[k]) {
+							if (Imap->msgids[j] == fts_msgs[k]) {
 								++is_in_list;
 							}
 						}
 					}
 					if (!is_in_list) {
-						IMAP->flags[j] = IMAP->flags[j] & ~IMAP_SELECTED;
+						Imap->flags[j] = Imap->flags[j] & ~IMAP_SELECTED;
 					}
 				}
 			}
 			else {		/* no hits on the index; disqualify every message */
-				for (j=0; j < IMAP->num_msgs; ++j) {
-					IMAP->flags[j] = IMAP->flags[j] & ~IMAP_SELECTED;
+				for (j=0; j < Imap->num_msgs; ++j) {
+					Imap->flags[j] = Imap->flags[j] & ~IMAP_SELECTED;
 				}
 			}
 			if (fts_msgs) {
@@ -603,24 +602,24 @@ void imap_do_search(int num_items, ConstStr *itemlist, int is_uid) {
 
 	/* Now go through the messages and apply all search criteria. */
 	buffer_output();
-	cprintf("* SEARCH ");
-	if (IMAP->num_msgs > 0)
-	 for (i = 0; i < IMAP->num_msgs; ++i)
-	  if (IMAP->flags[i] & IMAP_SELECTED) {
+	IAPuts("* SEARCH ");
+	if (Imap->num_msgs > 0)
+	 for (i = 0; i < Imap->num_msgs; ++i)
+	  if (Imap->flags[i] & IMAP_SELECTED) {
 		if (imap_do_search_msg(i+1, NULL, num_items, itemlist, is_uid)) {
 			if (num_results != 0) {
-				cprintf(" ");
+				IAPuts(" ");
 			}
 			if (is_uid) {
-				cprintf("%ld", IMAP->msgids[i]);
+				IAPrintf("%ld", Imap->msgids[i]);
 			}
 			else {
-				cprintf("%d", i+1);
+				IAPrintf("%d", i+1);
 			}
 			++num_results;
 		}
 	}
-	cprintf("\r\n");
+	IAPuts("\r\n");
 	unbuffer_output();
 }
 
@@ -632,7 +631,7 @@ void imap_search(int num_parms, ConstStr *Params) {
 	int i;
 
 	if (num_parms < 3) {
-		cprintf("%s BAD invalid parameters\r\n", Params[0].Key);
+		IReply("BAD invalid parameters");
 		return;
 	}
 
@@ -641,7 +640,7 @@ void imap_search(int num_parms, ConstStr *Params) {
 	}
 
 	imap_do_search(num_parms-2, &Params[2], 0);
-	cprintf("%s OK SEARCH completed\r\n", Params[0].Key);
+	IReply("OK SEARCH completed");
 }
 
 /*
@@ -651,7 +650,7 @@ void imap_uidsearch(int num_parms, ConstStr *Params) {
 	int i;
 
 	if (num_parms < 4) {
-		cprintf("%s BAD invalid parameters\r\n", Params[0].Key);
+		IReply("BAD invalid parameters");
 		return;
 	}
 
@@ -660,7 +659,7 @@ void imap_uidsearch(int num_parms, ConstStr *Params) {
 	}
 
 	imap_do_search(num_parms-3, &Params[3], 1);
-	cprintf("%s OK UID SEARCH completed\r\n", Params[0].Key);
+	IReply("OK UID SEARCH completed");
 }
 
 
