@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * This module handles self-service subscription/unsubscription to mail lists.
  *
  * Copyright (c) 2002-2009 by the citadel.org team
@@ -517,7 +515,7 @@ void do_confirm(char *room, char *token) {
 	 */
 	if (success) {
 		cprintf("%d %d operation(s) confirmed.\n", CIT_OK, success);
-		CtdlLogPrintf(CTDL_NOTICE, 
+		syslog(LOG_NOTICE, 
 			"Mailing list: %s %ssubscribed to %s with token %s\n", 
 			email, 
 			(!IsEmptyStr(address_to_unsubscribe)) ? "un" : "", 
@@ -586,6 +584,6 @@ CTDL_MODULE_INIT(listsub)
 		CtdlRegisterProtoHook(cmd_subs, "SUBS", "List subscribe/unsubscribe");
 	}
 	
-	/* return our Subversion id for the Log */
-	return "$Id$";
+	/* return our module name for the log */
+	return "listsub";
 }
