@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * Automatically copies the contents of a "New User Greetings" room to the
  * inbox of any new user upon account creation.
  *
@@ -115,6 +113,6 @@ CTDL_MODULE_INIT(newuser)
 		CtdlRegisterSessionHook(CopyNewUserGreetings, EVT_LOGIN);
 	}
 	
-	/* return our Subversion id for the Log */
-	return "$Id$";
+	/* return our module name for the log */
+	return "newuser";
 }
