@@ -1,23 +1,20 @@
 /*
- * $Id$
+ * This is an implementation of OpenID 2.0 relying party support in stateless mode.
  *
- * This is an implementation of OpenID 1.1 Relying Party support, in stateless mode.
+ * Copyright (c) 2007-2011 by the citadel.org team
  *
- * Copyright (c) 2007-2009 by the citadel.org team
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "sysdep.h"
@@ -45,24 +42,33 @@
 #include <string.h>
 #include <limits.h>
 #include <curl/curl.h>
+#include <expat.h>
 #include "ctdl_module.h"
 #include "config.h"
 #include "citserver.h"
 #include "user_ops.h"
 
 typedef struct _ctdl_openid {
-	StrBuf *claimed_id;
-	StrBuf *server;
+	StrBuf *op_url;			/* OpenID Provider Endpoint URL */
+	StrBuf *claimed_id;		/* Claimed Identifier */
 	int verified;
 	HashList *sreg_keys;
 } ctdl_openid;
 
+enum {
+	openid_disco_none,
+	openid_disco_xrds,
+	openid_disco_html
+};
+
+
 void Free_ctdl_openid(ctdl_openid **FreeMe)
 {
-	if (*FreeMe == NULL)
+	if (*FreeMe == NULL) {
 		return;
+	}
+	FreeStrBuf(&(*FreeMe)->op_url);
 	FreeStrBuf(&(*FreeMe)->claimed_id);
-	FreeStrBuf(&(*FreeMe)->server);
 	DeleteHash(&(*FreeMe)->sreg_keys);
 	free(*FreeMe);
 	*FreeMe = NULL;
@@ -76,7 +82,7 @@ void openid_cleanup_function(void) {
 	struct CitContext *CCC = CC;	/* CachedCitContext - performance boost */
 
 	if (CCC->openid_data != NULL) {
-		CtdlLogPrintf(CTDL_DEBUG, "Clearing OpenID session state\n");
+		syslog(LOG_DEBUG, "Clearing OpenID session state");
 		Free_ctdl_openid((ctdl_openid **) &CCC->openid_data);
 	}
 }
@@ -127,11 +133,11 @@ int attach_openid(struct ctdluser *who, StrBuf *claimed_id)
 		cdb_free(cdboi);
 
 		if (fetched_usernum == who->usernum) {
-			CtdlLogPrintf(CTDL_INFO, "%s already associated; no action is taken\n", ChrPtr(claimed_id));
+			syslog(LOG_INFO, "%s already associated; no action is taken", ChrPtr(claimed_id));
 			return(0);
 		}
 		else {
-			CtdlLogPrintf(CTDL_INFO, "%s already belongs to another user\n", ChrPtr(claimed_id));
+			syslog(LOG_INFO, "%s already belongs to another user", ChrPtr(claimed_id));
 			return(3);
 		}
 	}
@@ -150,7 +156,7 @@ int attach_openid(struct ctdluser *who, StrBuf *claimed_id)
 	snprintf(buf, sizeof buf, "User <%s> (#%ld) has claimed the OpenID URL %s\n",
 		 who->fullname, who->usernum, ChrPtr(claimed_id));
 	CtdlAideMessage(buf, "OpenID claim");
-	CtdlLogPrintf(CTDL_INFO, "%s", buf);
+	syslog(LOG_INFO, "%s", buf);
 	return(0);
 }
 
@@ -172,7 +178,6 @@ void openid_purge(struct ctdluser *usbuf) {
 	keys = NewHash(1, NULL);
 	if (!keys) return;
 
-
 	cdb_rewind(CDB_OPENID);
 	while (cdboi = cdb_next_item(CDB_OPENID), cdboi != NULL) {
 		if (cdboi->len > sizeof(long)) {
@@ -190,7 +195,7 @@ void openid_purge(struct ctdluser *usbuf) {
 	HashPos = GetNewHashPos(keys, 0);
 	while (GetNextHashPos(keys, HashPos, &len, &Key, &Value)!=0)
 	{
-		CtdlLogPrintf(CTDL_DEBUG, "Deleting associated OpenID <%s>\n", (char*)Value);
+		syslog(LOG_DEBUG, "Deleting associated OpenID <%s>", (char*)Value);
 		cdb_delete(CDB_OPENID, Value, strlen(Value));
 		/* note: don't free(Value) -- deleting the hash list will handle this for us */
 	}
@@ -199,7 +204,6 @@ void openid_purge(struct ctdluser *usbuf) {
 }
 
 
-
 /*
  * List the OpenIDs associated with the currently logged in account
  */
@@ -254,88 +258,6 @@ void cmd_oida(char *argbuf) {
 }
 
 
-/*
- * Attempt to register (populate the vCard) the currently-logged-in user
- * using the data from Simple Registration Extension, if present.
- */
-void populate_vcard_from_sreg(HashList *sreg_keys) {
-
-	struct vCard *v;
-	int pop = 0;			/* number of fields populated */
-	char *data = NULL;
-	char *postcode = NULL;
-	char *country = NULL;
-
-	if (!sreg_keys) return;
-	v = vcard_new();
-	if (!v) return;
-
-	if (GetHash(sreg_keys, "identity", 8, (void *) &data)) {
-		vcard_add_prop(v, "url;type=openid", data);
-		++pop;
-	}
-
-	if (GetHash(sreg_keys, "sreg.email", 10, (void *) &data)) {
-		vcard_add_prop(v, "email;internet", data);
-		++pop;
-	}
-
-	if (GetHash(sreg_keys, "sreg.nickname", 13, (void *) &data)) {
-		vcard_add_prop(v, "nickname", data);
-		++pop;
-	}
-
-	if (GetHash(sreg_keys, "sreg.fullname", 13, (void *) &data)) {
-		char n[256];
-		vcard_add_prop(v, "fn", data);
-		vcard_fn_to_n(n, data, sizeof n);
-		vcard_add_prop(v, "n", n);
-		++pop;
-	}
-
-	if (!GetHash(sreg_keys, "sreg.postcode", 13, (void *) &postcode)) {
-		postcode = NULL;
-	}
-
-	if (!GetHash(sreg_keys, "sreg.country", 12, (void *) &country)) {
-		country = NULL;
-	}
-
-	if (postcode || country) {
-		char adr[256];
-		snprintf(adr, sizeof adr, ";;;;;%s;%s",
-			(postcode ? postcode : ""),
-			(country ? country : "")
-		);
-		vcard_add_prop(v, "adr", adr);
-		++pop;
-	}
-
-	if (GetHash(sreg_keys, "sreg.dob", 8, (void *) &data)) {
-		vcard_add_prop(v, "bday", data);
-		++pop;
-	}
-
-	if (GetHash(sreg_keys, "sreg.gender", 11, (void *) &data)) {
-		vcard_add_prop(v, "x-funambol-gender", data);
-		++pop;
-	}
-
-	/* Only save the vCard if there is some useful data in it */
-	if (pop > 0) {
-		char *ser;
-		ser = vcard_serialize(v);
-		if (ser) {
-			CtdlWriteObject(USERCONFIGROOM,	"text/x-vcard",
-				ser, strlen(ser)+1, &CC->user, 0, 0, 0
-			);
-			free(ser);
-		}
-	}
-	vcard_free(v);
-}
-
-
 /*
  * Create a new user account, manually specifying the name, after successfully
  * verifying an OpenID (which will of course be attached to the account)
@@ -343,12 +265,7 @@ void populate_vcard_from_sreg(HashList *sreg_keys) {
 void cmd_oidc(char *argbuf) {
 	ctdl_openid *oiddata = (ctdl_openid *) CC->openid_data;
 
-	if (!oiddata) {
-		cprintf("%d You have not verified an OpenID yet.\n", ERROR);
-		return;
-	}
-
-	if (!oiddata->verified) {
+	if ( (!oiddata) || (!oiddata->verified) ) {
 		cprintf("%d You have not verified an OpenID yet.\n", ERROR);
 		return;
 	}
@@ -361,16 +278,11 @@ void cmd_oidc(char *argbuf) {
 	/* Now, if this logged us in, we have to attach the OpenID */
 	if (CC->logged_in) {
 		attach_openid(&CC->user, oiddata->claimed_id);
-		if (oiddata->sreg_keys != NULL) {
-			populate_vcard_from_sreg(oiddata->sreg_keys);
-		}
 	}
 
 }
 
 
-
-
 /*
  * Detach an OpenID from the currently logged in account
  */
@@ -408,37 +320,77 @@ void cmd_oidd(char *argbuf) {
 }
 
 
-
 /*
- * Attempt to auto-create a new Citadel account using the nickname from Simple Registration Extension
+ * Attempt to auto-create a new Citadel account using the nickname from Attribute Exchange
  */
-int openid_create_user_via_sreg(StrBuf *claimed_id, HashList *sreg_keys)
+int openid_create_user_via_ax(StrBuf *claimed_id, HashList *sreg_keys)
 {
-	char *desired_name = NULL;
+	char *nickname = NULL;
+	char *firstname = NULL;
+	char *lastname = NULL;
 	char new_password[32];
 	long len;
+	const char *Key;
+	void *Value;
 
 	if (config.c_auth_mode != AUTHMODE_NATIVE) return(1);
 	if (config.c_disable_newu) return(2);
 	if (CC->logged_in) return(3);
-	if (!GetHash(sreg_keys, "sreg.nickname", 13, (void *) &desired_name)) return(4);
 
-	CtdlLogPrintf(CTDL_DEBUG, "The desired account name is <%s>\n", desired_name);
+	HashPos *HashPos = GetNewHashPos(sreg_keys, 0);
+	while (GetNextHashPos(sreg_keys, HashPos, &len, &Key, &Value) != 0) {
+		syslog(LOG_DEBUG, "%s = %s", Key, (char *)Value);
 
-	len = cutuserkey(desired_name);
-	if (!CtdlGetUser(&CC->user, desired_name)) {
-		CtdlLogPrintf(CTDL_DEBUG, "<%s> is already taken by another user.\n", desired_name);
+		if (cbmstrcasestr(Key, "value.nickname") != NULL) {
+			nickname = (char *)Value;
+		}
+		else if ( (nickname == NULL) && (cbmstrcasestr(Key, "value.nickname") != NULL)) {
+			nickname = (char *)Value;
+		}
+		else if (cbmstrcasestr(Key, "value.firstname") != NULL) {
+			firstname = (char *)Value;
+		}
+		else if (cbmstrcasestr(Key, "value.lastname") != NULL) {
+			lastname = (char *)Value;
+		}
+
+	}
+	DeleteHashPos(&HashPos);
+
+	if (nickname == NULL) {
+		if ((firstname != NULL) || (lastname != NULL)) {
+			char fullname[1024] = "";
+			if (firstname) strcpy(fullname, firstname);
+			if (firstname && lastname) strcat(fullname, " ");
+			if (lastname) strcat(fullname, lastname);
+			nickname = fullname;
+		}
+	}
+
+	if (nickname == NULL) {
+		return(4);
+	}
+	syslog(LOG_DEBUG, "The desired account name is <%s>", nickname);
+
+	len = cutuserkey(nickname);
+	if (!CtdlGetUser(&CC->user, nickname)) {
+		syslog(LOG_DEBUG, "<%s> is already taken by another user.", nickname);
 		memset(&CC->user, 0, sizeof(struct ctdluser));
 		return(5);
 	}
 
 	/* The desired account name is available.  Create the account and log it in! */
-	if (create_user(desired_name, len, 1)) return(6);
+	if (create_user(nickname, len, 1)) return(6);
 
+	/* Generate a random password.
+	 * The user doesn't care what the password is since he is using OpenID.
+	 */
 	snprintf(new_password, sizeof new_password, "%08lx%08lx", random(), random());
 	CtdlSetPassword(new_password);
+
+	/* Now attach the verified OpenID to this account. */
 	attach_openid(&CC->user, claimed_id);
-	populate_vcard_from_sreg(sreg_keys);
+
 	return(0);
 }
 
@@ -487,7 +439,7 @@ int login_via_openid(StrBuf *claimed_id)
  */
 void extract_link(StrBuf *target_buf, const char *rel, long repllen, StrBuf *source_buf)
 {
-	int len, i;
+	int i;
 	const char *ptr;
 	const char *href_start = NULL;
 	const char *href_end = NULL;
@@ -510,8 +462,6 @@ void extract_link(StrBuf *target_buf, const char *rel, long repllen, StrBuf *sou
 		if (link_tag_end == NULL)
 			break;
 		for (i=0; i < 1; i++ ){
-			len = link_tag_end - link_tag_start;
-
 			rel_start = cbmstrcasestr(link_tag_start, "rel=");
 			if ((rel_start == NULL) ||
 			    (rel_start > link_tag_end)) 
@@ -552,56 +502,295 @@ void extract_link(StrBuf *target_buf, const char *rel, long repllen, StrBuf *sou
 
 
 /*
- * Begin an HTTP fetch (returns number of bytes actually fetched, or -1 for error) using libcurl.
- *
- * If 'normalize_len' is nonzero, the caller is specifying the buffer size of 'url', and is
- * requesting that the effective (normalized) URL be copied back to it.
+ * Wrapper for curl_easy_init() that includes the options common to all calls
+ * used in this module. 
  */
-int fetch_http(StrBuf *url, StrBuf **target_buf)
-{
-	StrBuf *ReplyBuf;
+CURL *ctdl_openid_curl_easy_init(char *errmsg) {
 	CURL *curl;
-	CURLcode res;
-	char errmsg[1024] = "";
-	char *effective_url = NULL;
-
-	if (StrLength(url) <=0 ) return(-1);
-	ReplyBuf = *target_buf = NewStrBuf ();
-	if (ReplyBuf == 0) return(-1);
 
 	curl = curl_easy_init();
 	if (!curl) {
-		CtdlLogPrintf(CTDL_ALERT, "Unable to initialize libcurl.\n");
-		return(-1);
+		return(curl);
 	}
 
-	curl_easy_setopt(curl, CURLOPT_URL, ChrPtr(url));
 	curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0);
 	curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0);
 
-	curl_easy_setopt(curl, CURLOPT_WRITEDATA, ReplyBuf);
-	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, CurlFillStrBuf_callback);
-
-	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errmsg);
+	if (errmsg) {
+		curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errmsg);
+	}
 	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
 #ifdef CURLOPT_HTTP_CONTENT_DECODING
 	curl_easy_setopt(curl, CURLOPT_HTTP_CONTENT_DECODING, 1);
 	curl_easy_setopt(curl, CURLOPT_ENCODING, "");
 #endif
 	curl_easy_setopt(curl, CURLOPT_USERAGENT, CITADEL);
-	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 180);		/* die after 180 seconds */
-	if (!IsEmptyStr(config.c_ip_addr)) {
+	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 30);		/* die after 30 seconds */
+
+	if (
+		(!IsEmptyStr(config.c_ip_addr))
+		&& (strcmp(config.c_ip_addr, "*"))
+		&& (strcmp(config.c_ip_addr, "::"))
+		&& (strcmp(config.c_ip_addr, "0.0.0.0"))
+	) {
 		curl_easy_setopt(curl, CURLOPT_INTERFACE, config.c_ip_addr);
 	}
-	res = curl_easy_perform(curl);
-	if (res) {
-		CtdlLogPrintf(CTDL_DEBUG, "fetch_http() libcurl error %d: %s\n", res, errmsg);
+
+	return(curl);
+}
+
+
+struct xrds {
+	StrBuf *CharData;
+	int nesting_level;
+	int in_xrd;
+	int current_service_priority;
+	int selected_service_priority;
+	StrBuf *current_service_uri;
+	StrBuf *selected_service_uri;
+	int current_service_is_oid2auth;
+};
+
+
+void xrds_xml_start(void *data, const char *supplied_el, const char **attr) {
+	struct xrds *xrds = (struct xrds *) data;
+	int i;
+
+	++xrds->nesting_level;
+
+	if (!strcasecmp(supplied_el, "XRD")) {
+		++xrds->in_xrd;
 	}
-	curl_easy_getinfo(curl, CURLINFO_EFFECTIVE_URL, &effective_url);
-	StrBufPlain(url, effective_url, -1);
-	
+
+	else if (!strcasecmp(supplied_el, "service")) {
+		xrds->current_service_priority = 0;
+		xrds->current_service_is_oid2auth = 0;
+		for (i=0; attr[i] != NULL; i+=2) {
+			if (!strcasecmp(attr[i], "priority")) {
+				xrds->current_service_priority = atoi(attr[i+1]);
+			}
+		}
+	}
+
+	FlushStrBuf(xrds->CharData);
+}
+
+
+void xrds_xml_end(void *data, const char *supplied_el) {
+	struct xrds *xrds = (struct xrds *) data;
+
+	--xrds->nesting_level;
+
+	if (!strcasecmp(supplied_el, "XRD")) {
+		--xrds->in_xrd;
+	}
+
+	else if (!strcasecmp(supplied_el, "type")) {
+		if (	(xrds->in_xrd)
+			&& (!strcasecmp(ChrPtr(xrds->CharData), "http://specs.openid.net/auth/2.0/server"))
+		) {
+			xrds->current_service_is_oid2auth = 1;
+		}
+		if (	(xrds->in_xrd)
+			&& (!strcasecmp(ChrPtr(xrds->CharData), "http://specs.openid.net/auth/2.0/signon"))
+		) {
+			xrds->current_service_is_oid2auth = 1;
+			/* FIXME in this case, the Claimed ID should be considered immutable */
+		}
+	}
+
+	else if (!strcasecmp(supplied_el, "uri")) {
+		if (xrds->in_xrd) {
+			FlushStrBuf(xrds->current_service_uri);
+			StrBufAppendBuf(xrds->current_service_uri, xrds->CharData, 0);
+		}
+	}
+
+	else if (!strcasecmp(supplied_el, "service")) {
+		if (	(xrds->in_xrd)
+			&& (xrds->current_service_priority < xrds->selected_service_priority)
+			&& (xrds->current_service_is_oid2auth)
+		) {
+			xrds->selected_service_priority = xrds->current_service_priority;
+			FlushStrBuf(xrds->selected_service_uri);
+			StrBufAppendBuf(xrds->selected_service_uri, xrds->current_service_uri, 0);
+		}
+
+	}
+
+	FlushStrBuf(xrds->CharData);
+}
+
+
+void xrds_xml_chardata(void *data, const XML_Char *s, int len) {
+	struct xrds *xrds = (struct xrds *) data;
+
+	StrBufAppendBufPlain (xrds->CharData, s, len, 0);
+}
+
+
+/*
+ * Parse an XRDS document.
+ * If an OpenID Provider URL is discovered, op_url to that value and return nonzero.
+ * If nothing useful happened, return 0.
+ */
+int parse_xrds_document(StrBuf *ReplyBuf) {
+	ctdl_openid *oiddata = (ctdl_openid *) CC->openid_data;
+	struct xrds xrds;
+	int return_value = 0;
+
+	memset(&xrds, 0, sizeof (struct xrds));
+	xrds.selected_service_priority = INT_MAX;
+	xrds.CharData = NewStrBuf();
+	xrds.current_service_uri = NewStrBuf();
+	xrds.selected_service_uri = NewStrBuf();
+	XML_Parser xp = XML_ParserCreate(NULL);
+	if (xp) {
+		XML_SetUserData(xp, &xrds);
+		XML_SetElementHandler(xp, xrds_xml_start, xrds_xml_end);
+		XML_SetCharacterDataHandler(xp, xrds_xml_chardata);
+		XML_Parse(xp, ChrPtr(ReplyBuf), StrLength(ReplyBuf), 0);
+		XML_Parse(xp, "", 0, 1);
+		XML_ParserFree(xp);
+	}
+	else {
+		syslog(LOG_ALERT, "Cannot create XML parser");
+	}
+
+	if (xrds.selected_service_priority < INT_MAX) {
+		if (oiddata->op_url == NULL) {
+			oiddata->op_url = NewStrBuf();
+		}
+		FlushStrBuf(oiddata->op_url);
+		StrBufAppendBuf(oiddata->op_url, xrds.selected_service_uri, 0);
+		return_value = openid_disco_xrds;
+	}
+
+	FreeStrBuf(&xrds.CharData);
+	FreeStrBuf(&xrds.current_service_uri);
+	FreeStrBuf(&xrds.selected_service_uri);
+
+	return(return_value);
+}
+
+
+/*
+ * Callback function for perform_openid2_discovery()
+ * We're interested in the X-XRDS-Location: header.
+ */
+size_t yadis_headerfunction(void *ptr, size_t size, size_t nmemb, void *userdata) {
+	char hdr[1024];
+	StrBuf **x_xrds_location = (StrBuf **) userdata;
+
+	memcpy(hdr, ptr, (size*nmemb));
+	hdr[size*nmemb] = 0;
+
+	if (!strncasecmp(hdr, "X-XRDS-Location:", 16)) {
+		*x_xrds_location = NewStrBufPlain(&hdr[16], ((size*nmemb)-16));
+		StrBufTrim(*x_xrds_location);
+	}
+
+	return(size * nmemb);
+}
+
+
+/* Attempt to perform Yadis discovery as specified in Yadis 1.0 section 6.2.5.
+ * 
+ * If Yadis fails, we then attempt HTML discovery using the same document.
+ *
+ * If successful, returns nonzero and calls parse_xrds_document() to act upon the received data.
+ * If fails, returns 0 and does nothing else.
+ */
+int perform_openid2_discovery(StrBuf *SuppliedURL) {
+	ctdl_openid *oiddata = (ctdl_openid *) CC->openid_data;
+	int docbytes = (-1);
+	StrBuf *ReplyBuf = NULL;
+	int return_value = 0;
+	CURL *curl;
+	CURLcode result;
+	char errmsg[1024] = "";
+	struct curl_slist *my_headers = NULL;
+	StrBuf *x_xrds_location = NULL;
+
+	if (!SuppliedURL) return(0);
+	syslog(LOG_DEBUG, "perform_openid2_discovery(%s)", ChrPtr(SuppliedURL));
+	if (StrLength(SuppliedURL) == 0) return(0);
+
+	ReplyBuf = NewStrBuf();
+	if (!ReplyBuf) return(0);
+
+	curl = ctdl_openid_curl_easy_init(errmsg);
+	if (!curl) return(0);
+
+	curl_easy_setopt(curl, CURLOPT_URL, ChrPtr(SuppliedURL));
+	curl_easy_setopt(curl, CURLOPT_WRITEDATA, ReplyBuf);
+	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, CurlFillStrBuf_callback);
+
+	my_headers = curl_slist_append(my_headers, "Accept:");	/* disable the default Accept: header */
+	my_headers = curl_slist_append(my_headers, "Accept: application/xrds+xml");
+	curl_easy_setopt(curl, CURLOPT_HTTPHEADER, my_headers);
+
+	curl_easy_setopt(curl, CURLOPT_WRITEHEADER, &x_xrds_location);
+	curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, yadis_headerfunction);
+
+	result = curl_easy_perform(curl);
+	if (result) {
+		syslog(LOG_DEBUG, "libcurl error %d: %s", result, errmsg);
+	}
+	curl_slist_free_all(my_headers);
 	curl_easy_cleanup(curl);
-	return StrLength(ReplyBuf);
+	docbytes = StrLength(ReplyBuf);
+
+	/*
+	 * The response from the server will be one of:
+	 * 
+	 * Option 1: An HTML document with a <head> element that includes a <meta> element with http-equiv
+	 * attribute, X-XRDS-Location,
+	 *
+	 * Does any provider actually do this?  If so then we will implement it in the future.
+	 */
+
+	/*
+	 * Option 2: HTTP response-headers that include an X-XRDS-Location response-header,
+	 *           together with a document.
+	 * Option 3: HTTP response-headers only, which MAY include an X-XRDS-Location response-header,
+	 *           a contenttype response-header specifying MIME media type,
+	 *           application/xrds+xml, or both.
+	 *
+	 * If the X-XRDS-Location header was delivered, we know about it at this point...
+	 */
+	if (	(x_xrds_location)
+		&& (strcmp(ChrPtr(x_xrds_location), ChrPtr(SuppliedURL)))
+	) {
+		syslog(LOG_DEBUG, "X-XRDS-Location: %s ... recursing!", ChrPtr(x_xrds_location));
+		return_value = perform_openid2_discovery(x_xrds_location);
+		FreeStrBuf(&x_xrds_location);
+	}
+
+	/*
+	 * Option 4: the returned web page may *be* an XRDS document.  Try to parse it.
+	 */
+	if ( (return_value == 0) && (docbytes >= 0)) {
+		return_value = parse_xrds_document(ReplyBuf);
+	}
+
+	/*
+	 * Option 5: if all else fails, attempt HTML based discovery.
+	 */
+	if ( (return_value == 0) && (docbytes >= 0)) {
+		if (oiddata->op_url == NULL) {
+			oiddata->op_url = NewStrBuf();
+		}
+		extract_link(oiddata->op_url, HKEY("openid2.provider"), ReplyBuf);
+		if (StrLength(oiddata->op_url) > 0) {
+			return_value = openid_disco_html;
+		}
+	}
+
+	if (ReplyBuf != NULL) {
+		FreeStrBuf(&ReplyBuf);
+	}
+	return(return_value);
 }
 
 
@@ -609,21 +798,20 @@ int fetch_http(StrBuf *url, StrBuf **target_buf)
  * Setup an OpenID authentication
  */
 void cmd_oids(char *argbuf) {
+	struct CitContext *CCC = CC;	/* CachedCitContext - performance boost */
 	const char *Pos = NULL;
 	StrBuf *ArgBuf = NULL;
 	StrBuf *ReplyBuf = NULL;
 	StrBuf *return_to = NULL;
-	StrBuf *trust_root = NULL;
-	StrBuf *openid_delegate = NULL;
 	StrBuf *RedirectUrl = NULL;
-	int i;
-	struct CitContext *CCC = CC;	/* CachedCitContext - performance boost */
 	ctdl_openid *oiddata;
+	int discovery_succeeded = 0;
 
 	Free_ctdl_openid ((ctdl_openid**)&CCC->openid_data);
 
 	CCC->openid_data = oiddata = malloc(sizeof(ctdl_openid));
 	if (oiddata == NULL) {
+		syslog(LOG_ALERT, "malloc() failed: %s", strerror(errno));
 		cprintf("%d malloc failed\n", ERROR + INTERNAL_ERROR);
 		return;
 	}
@@ -632,84 +820,106 @@ void cmd_oids(char *argbuf) {
 
 	ArgBuf = NewStrBufPlain(argbuf, -1);
 
+	oiddata->verified = 0;
 	oiddata->claimed_id = NewStrBufPlain(NULL, StrLength(ArgBuf));
-	trust_root = NewStrBufPlain(NULL, StrLength(ArgBuf));
 	return_to = NewStrBufPlain(NULL, StrLength(ArgBuf));
 
 	StrBufExtract_NextToken(oiddata->claimed_id, ArgBuf, &Pos, '|');
 	StrBufExtract_NextToken(return_to, ArgBuf, &Pos, '|');
-	StrBufExtract_NextToken(trust_root, ArgBuf, &Pos, '|');
+
+	syslog(LOG_DEBUG, "User-Supplied Identifier is: %s", ChrPtr(oiddata->claimed_id));
+
+	/********** OpenID 2.0 section 7.3 - Discovery **********/
+
+	/* Section 7.3.1 says we have to attempt XRI based discovery.
+	 * No one is using this, no one is asking for it, no one wants it.
+	 * So we're not even going to bother attempting this mode.
+	 */
+
+	/* Attempt section 7.3.2 (Yadis discovery) and section 7.3.3 (HTML discovery);
+	 */
+	discovery_succeeded = perform_openid2_discovery(oiddata->claimed_id);
+
+	if (discovery_succeeded == 0) {
+		cprintf("%d There is no OpenID identity provider at this location.\n", ERROR);
+	}
+
+	else {
+		/*
+		 * If we get to this point we are in possession of a valid OpenID Provider URL.
+		 */
+		syslog(LOG_DEBUG, "OP URI '%s' discovered using method %d",
+			ChrPtr(oiddata->op_url),
+			discovery_succeeded
+		);
+
+		/* We have to "normalize" our Claimed ID otherwise it will cause some OP's to barf */
+		if (cbmstrcasestr(ChrPtr(oiddata->claimed_id), "://") == NULL) {
+			StrBuf *cid = oiddata->claimed_id;
+			oiddata->claimed_id = NewStrBufPlain(HKEY("http://"));
+			StrBufAppendBuf(oiddata->claimed_id, cid, 0);
+			FreeStrBuf(&cid);
+		}
+
+		/*
+		 * OpenID 2.0 section 9: request authentication
+		 * Assemble a URL to which the user-agent will be redirected.
+		 */
 	
-	oiddata->verified = 0;
+		RedirectUrl = NewStrBufDup(oiddata->op_url);
 
-	i = fetch_http(oiddata->claimed_id, &ReplyBuf);
-	CtdlLogPrintf(CTDL_DEBUG, "Normalized URL and Claimed ID is: %s\n", 
-		      ChrPtr(oiddata->claimed_id));
-	if ((StrLength(ReplyBuf) > 0) && (i > 0)) {
+		StrBufAppendBufPlain(RedirectUrl, HKEY("?openid.ns="), 0);
+		StrBufUrlescAppend(RedirectUrl, NULL, "http://specs.openid.net/auth/2.0");
 
-		openid_delegate = NewStrBuf();
-		oiddata->server = NewStrBuf();
-		extract_link(oiddata->server, HKEY("openid.server"), ReplyBuf);
-		extract_link(openid_delegate, HKEY("openid.delegate"), ReplyBuf);
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.mode=checkid_setup"), 0);
 
-		if (StrLength(oiddata->server) == 0) {
-			cprintf("%d There is no OpenID identity provider at this URL.\n", ERROR);
-			FreeStrBuf(&ArgBuf);
-			FreeStrBuf(&ReplyBuf);
-			FreeStrBuf(&return_to);
-			FreeStrBuf(&trust_root);
-			FreeStrBuf(&openid_delegate);
-			FreeStrBuf(&RedirectUrl);
-			return;
-		}
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.claimed_id="), 0);
+		StrBufUrlescAppend(RedirectUrl, oiddata->claimed_id, NULL);
 
-		/* Empty delegate is legal; we just use the openid_url instead */
-		if (StrLength(openid_delegate) == 0) {
-			StrBufPlain(openid_delegate, SKEY(oiddata->claimed_id));
-		}
-
-		/* Assemble a URL to which the user-agent will be redirected. */
-
-		RedirectUrl = NewStrBufDup(oiddata->server);
-
-		StrBufAppendBufPlain(RedirectUrl, HKEY("?openid.mode=checkid_setup"
-						       "&openid.identity="), 0);
-		StrBufUrlescAppend(RedirectUrl, openid_delegate, NULL);
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.identity="), 0);
+		StrBufUrlescAppend(RedirectUrl, oiddata->claimed_id, NULL);
 
+		/* return_to tells the provider how to complete the round trip back to our site */
 		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.return_to="), 0);
 		StrBufUrlescAppend(RedirectUrl, return_to, NULL);
 
-		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.trust_root="), 0);
-		StrBufUrlescAppend(RedirectUrl, trust_root, NULL);
+		/* Attribute Exchange
+		 * See:
+		 *	http://openid.net/specs/openid-attribute-exchange-1_0.html
+		 *	http://code.google.com/apis/accounts/docs/OpenID.html#endpoint
+		 * 	http://test-id.net/OP/AXFetch.aspx
+		 */
 
-		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.sreg.optional="), 0);
-		StrBufUrlescAppend(RedirectUrl, NULL, "nickname,email,fullname,postcode,country,dob,gender");
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.ns.ax="), 0);
+		StrBufUrlescAppend(RedirectUrl, NULL, "http://openid.net/srv/ax/1.0");
 
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.ax.mode=fetch_request"), 0);
+
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.ax.required=firstname,lastname,friendly,nickname"), 0);
+
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.ax.type.firstname="), 0);
+		StrBufUrlescAppend(RedirectUrl, NULL, "http://axschema.org/namePerson/first");
+
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.ax.type.lastname="), 0);
+		StrBufUrlescAppend(RedirectUrl, NULL, "http://axschema.org/namePerson/last");
+
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.ax.type.friendly="), 0);
+		StrBufUrlescAppend(RedirectUrl, NULL, "http://axschema.org/namePerson/friendly");
+
+		StrBufAppendBufPlain(RedirectUrl, HKEY("&openid.ax.type.nickname="), 0);
+		StrBufUrlescAppend(RedirectUrl, NULL, "http://axschema.org/namePerson/nickname");
+
+		syslog(LOG_DEBUG, "OpenID: redirecting client to %s", ChrPtr(RedirectUrl));
 		cprintf("%d %s\n", CIT_OK, ChrPtr(RedirectUrl));
-		
-		FreeStrBuf(&ArgBuf);
-		FreeStrBuf(&ReplyBuf);
-		FreeStrBuf(&return_to);
-		FreeStrBuf(&trust_root);
-		FreeStrBuf(&openid_delegate);
-		FreeStrBuf(&RedirectUrl);
-
-		return;
 	}
+	
 	FreeStrBuf(&ArgBuf);
 	FreeStrBuf(&ReplyBuf);
 	FreeStrBuf(&return_to);
-	FreeStrBuf(&trust_root);
-	FreeStrBuf(&openid_delegate);
 	FreeStrBuf(&RedirectUrl);
-
-	cprintf("%d Unable to fetch OpenID URL\n", ERROR);
 }
 
 
-
-
-
 /*
  * Finalize an OpenID authentication
  */
@@ -719,14 +929,16 @@ void cmd_oidf(char *argbuf) {
 	char thiskey[1024];
 	char thisdata[1024];
 	HashList *keys = NULL;
+	const char *Key;
+	void *Value;
 	ctdl_openid *oiddata = (ctdl_openid *) CC->openid_data;
 
 	if (oiddata == NULL) {
 		cprintf("%d run OIDS first.\n", ERROR + INTERNAL_ERROR);
 		return;
 	}
-	if (StrLength(oiddata->server) == 0){
-		cprintf("%d need a remote server to authenticate against\n", ERROR + ILLEGAL_VALUE);
+	if (StrLength(oiddata->op_url) == 0){
+		cprintf("%d No OpenID Endpoint URL has been obtained.\n", ERROR + ILLEGAL_VALUE);
 		return;
 	}
 	keys = NewHash(1, NULL);
@@ -738,116 +950,91 @@ void cmd_oidf(char *argbuf) {
 
 	while (client_getln(buf, sizeof buf), strcmp(buf, "000")) {
 		len = extract_token(thiskey, buf, 0, '|', sizeof thiskey);
-		if (len < 0)
+		if (len < 0) {
 			len = sizeof(thiskey) - 1;
+		}
 		extract_token(thisdata, buf, 1, '|', sizeof thisdata);
-		CtdlLogPrintf(CTDL_DEBUG, "%s: [%d] %s\n", thiskey, strlen(thisdata), thisdata);
 		Put(keys, thiskey, len, strdup(thisdata), NULL);
 	}
 
+	/* Check to see if this is a correct response.
+	 * Start with verified=1 but then set it to 0 if anything looks wrong.
+	 */
+	oiddata->verified = 1;
 
-	/* Now that we have all of the parameters, we have to validate the signature against the server */
-	CtdlLogPrintf(CTDL_DEBUG, "About to validate the signature...\n");
+	char *openid_ns = NULL;
+	if (	(!GetHash(keys, "ns", 2, (void *) &openid_ns))
+		|| (strcasecmp(openid_ns, "http://specs.openid.net/auth/2.0"))
+	) {
+		syslog(LOG_DEBUG, "This is not an an OpenID assertion");
+		oiddata->verified = 0;
+	}
+
+	char *openid_mode = NULL;
+	if (	(!GetHash(keys, "mode", 4, (void *) &openid_mode))
+		|| (strcasecmp(openid_mode, "id_res"))
+	) {
+		oiddata->verified = 0;
+	}
+
+	char *openid_claimed_id = NULL;
+	if (GetHash(keys, "claimed_id", 10, (void *) &openid_claimed_id)) {
+		FreeStrBuf(&oiddata->claimed_id);
+		oiddata->claimed_id = NewStrBufPlain(openid_claimed_id, -1);
+		syslog(LOG_DEBUG, "Provider is asserting the Claimed ID '%s'", ChrPtr(oiddata->claimed_id));
+	}
+
+	/* Validate the assertion against the server */
+	syslog(LOG_DEBUG, "Validating...");
 
 	CURL *curl;
 	CURLcode res;
 	struct curl_httppost *formpost = NULL;
 	struct curl_httppost *lastptr = NULL;
 	char errmsg[1024] = "";
-	char *o_assoc_handle = NULL;
-	char *o_sig = NULL;
-	char *o_signed = NULL;
-	int num_signed_values;
-	int i;
-	char k_keyname[128];
-	char k_o_keyname[128];
-	char *k_value = NULL;
-	StrBuf *ReplyBuf;
+	StrBuf *ReplyBuf = NewStrBuf();
 
 	curl_formadd(&formpost, &lastptr,
 		CURLFORM_COPYNAME,	"openid.mode",
 		CURLFORM_COPYCONTENTS,	"check_authentication",
-		CURLFORM_END);
-	CtdlLogPrintf(CTDL_DEBUG, "%25s : %s\n", "openid.mode", "check_authentication");
+		CURLFORM_END
+	);
 
-	if (GetHash(keys, "assoc_handle", 12, (void *) &o_assoc_handle)) {
-		curl_formadd(&formpost, &lastptr,
-			CURLFORM_COPYNAME,	"openid.assoc_handle",
-			CURLFORM_COPYCONTENTS,	o_assoc_handle,
-			CURLFORM_END);
-		CtdlLogPrintf(CTDL_DEBUG, "%25s : %s\n", "openid.assoc_handle", o_assoc_handle);
-	}
-
-	if (GetHash(keys, "sig", 3, (void *) &o_sig)) {
-		curl_formadd(&formpost, &lastptr,
-			CURLFORM_COPYNAME,	"openid.sig",
-			CURLFORM_COPYCONTENTS,	o_sig,
-			CURLFORM_END);
-			CtdlLogPrintf(CTDL_DEBUG, "%25s : %s\n", "openid.sig", o_sig);
-	}
-
-	if (GetHash(keys, "signed", 6, (void *) &o_signed)) {
-		curl_formadd(&formpost, &lastptr,
-			CURLFORM_COPYNAME,	"openid.signed",
-			CURLFORM_COPYCONTENTS,	o_signed,
-			CURLFORM_END);
-		CtdlLogPrintf(CTDL_DEBUG, "%25s : %s\n", "openid.signed", o_signed);
-
-		num_signed_values = num_tokens(o_signed, ',');
-		for (i=0; i<num_signed_values; ++i) {
-			extract_token(k_keyname, o_signed, i, ',', sizeof k_keyname);
-			if (strcasecmp(k_keyname, "mode")) {	// work around phpMyID bug
-				if (GetHash(keys, k_keyname, strlen(k_keyname), (void *) &k_value)) {
-					snprintf(k_o_keyname, sizeof k_o_keyname, "openid.%s", k_keyname);
-					curl_formadd(&formpost, &lastptr,
-						CURLFORM_COPYNAME,	k_o_keyname,
-						CURLFORM_COPYCONTENTS,	k_value,
-						CURLFORM_END);
-					CtdlLogPrintf(CTDL_DEBUG, "%25s : %s\n", k_o_keyname, k_value);
-				}
-				else {
-					CtdlLogPrintf(CTDL_INFO, "OpenID: signed field '%s' is missing\n",
-						k_keyname);
-				}
-			}
+	HashPos *HashPos = GetNewHashPos(keys, 0);
+	while (GetNextHashPos(keys, HashPos, &len, &Key, &Value) != 0) {
+		if (strcasecmp(Key, "mode")) {
+			char k_o_keyname[1024];
+			snprintf(k_o_keyname, sizeof k_o_keyname, "openid.%s", (const char *)Key);
+			curl_formadd(&formpost, &lastptr,
+				CURLFORM_COPYNAME,	k_o_keyname,
+				CURLFORM_COPYCONTENTS,	(char *)Value,
+				CURLFORM_END
+			);
 		}
 	}
-	
-	ReplyBuf = NewStrBuf();
-
-	curl = curl_easy_init();
-	curl_easy_setopt(curl, CURLOPT_URL, ChrPtr(oiddata->server));
-	curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0);
-	curl_easy_setopt(curl, CURLOPT_SSL_VERIFYHOST, 0);
+	DeleteHashPos(&HashPos);
 
+	curl = ctdl_openid_curl_easy_init(errmsg);
+	curl_easy_setopt(curl, CURLOPT_URL, ChrPtr(oiddata->op_url));
 	curl_easy_setopt(curl, CURLOPT_WRITEDATA, ReplyBuf);
 	curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, CurlFillStrBuf_callback);
 	curl_easy_setopt(curl, CURLOPT_HTTPPOST, formpost);
-	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errmsg);
-	curl_easy_setopt(curl, CURLOPT_FOLLOWLOCATION, 1);
-#ifdef CURLOPT_HTTP_CONTENT_DECODING
-	curl_easy_setopt(curl, CURLOPT_HTTP_CONTENT_DECODING, 1);
-	curl_easy_setopt(curl, CURLOPT_ENCODING, "");
-#endif
-	curl_easy_setopt(curl, CURLOPT_USERAGENT, CITADEL);
-	curl_easy_setopt(curl, CURLOPT_TIMEOUT, 180);		/* die after 180 seconds */
-	if (!IsEmptyStr(config.c_ip_addr)) {
-		curl_easy_setopt(curl, CURLOPT_INTERFACE, config.c_ip_addr);
-	}
 
 	res = curl_easy_perform(curl);
 	if (res) {
-		CtdlLogPrintf(CTDL_DEBUG, "cmd_oidf() libcurl error %d: %s\n", res, errmsg);
+		syslog(LOG_DEBUG, "cmd_oidf() libcurl error %d: %s", res, errmsg);
+		oiddata->verified = 0;
 	}
 	curl_easy_cleanup(curl);
 	curl_formfree(formpost);
 
-	if (cbmstrcasestr(ChrPtr(ReplyBuf), "is_valid:true")) {
-		oiddata->verified = 1;
+	/* syslog(LOG_DEBUG, "Validation reply: \n%s", ChrPtr(ReplyBuf)); */
+	if (cbmstrcasestr(ChrPtr(ReplyBuf), "is_valid:true") == NULL) {
+		oiddata->verified = 0;
 	}
 	FreeStrBuf(&ReplyBuf);
 
-	CtdlLogPrintf(CTDL_DEBUG, "Authentication %s.\n", (oiddata->verified ? "succeeded" : "failed") );
+	syslog(LOG_DEBUG, "OpenID authentication %s", (oiddata->verified ? "succeeded" : "failed") );
 
 	/* Respond to the client */
 
@@ -857,11 +1044,11 @@ void cmd_oidf(char *argbuf) {
 		if (CC->logged_in) {
 			if (attach_openid(&CC->user, oiddata->claimed_id) == 0) {
 				cprintf("attach\n");
-				CtdlLogPrintf(CTDL_DEBUG, "OpenID attach succeeded\n");
+				syslog(LOG_DEBUG, "OpenID attach succeeded");
 			}
 			else {
 				cprintf("fail\n");
-				CtdlLogPrintf(CTDL_DEBUG, "OpenID attach failed\n");
+				syslog(LOG_DEBUG, "OpenID attach failed");
 			}
 		}
 
@@ -878,7 +1065,7 @@ void cmd_oidf(char *argbuf) {
 			if (login_via_openid(oiddata->claimed_id) == 0) {
 				cprintf("authenticate\n%s\n%s\n", CC->user.fullname, CC->user.password);
 				logged_in_response();
-				CtdlLogPrintf(CTDL_DEBUG, "Logged in using previously claimed OpenID\n");
+				syslog(LOG_DEBUG, "Logged in using previously claimed OpenID");
 			}
 
 			/*
@@ -887,16 +1074,16 @@ void cmd_oidf(char *argbuf) {
 			 */
 			else if (config.c_disable_newu) {
 				cprintf("fail\n");
-				CtdlLogPrintf(CTDL_DEBUG, "Creating user failed due to local policy\n");
+				syslog(LOG_DEBUG, "Creating user failed due to local policy");
 			}
 
 			/*
-			 * New user whose OpenID is verified and Simple Registration Extension is in use?
+			 * New user whose OpenID is verified and Attribute Exchange gave us a name?
 			 */
-			else if (openid_create_user_via_sreg(oiddata->claimed_id, keys) == 0) {
+			else if (openid_create_user_via_ax(oiddata->claimed_id, keys) == 0) {
 				cprintf("authenticate\n%s\n%s\n", CC->user.fullname, CC->user.password);
 				logged_in_response();
-				CtdlLogPrintf(CTDL_DEBUG, "Successfully auto-created new user\n");
+				syslog(LOG_DEBUG, "Successfully auto-created new user");
 			}
 
 			/*
@@ -913,7 +1100,7 @@ void cmd_oidf(char *argbuf) {
 				else {
 					cprintf("\n");
 				}
-				CtdlLogPrintf(CTDL_DEBUG, "The desired Simple Registration name is already taken.\n");
+				syslog(LOG_DEBUG, "The desired display name is already taken.");
 			}
 		}
 	}
@@ -956,8 +1143,9 @@ CTDL_MODULE_INIT(openid_rp)
 		}
 		CtdlRegisterSessionHook(openid_cleanup_function, EVT_LOGOUT);
 		CtdlRegisterUserHook(openid_purge, EVT_PURGEUSER);
+		openid_level_supported = 1;	/* This module supports OpenID 1.0 only */
 	}
 
-	/* return our Subversion id for the Log */
-	return "$Id$";
+	/* return our module name for the log */
+	return "openid_rp";
 }
