@@ -1,6 +1,4 @@
 /*
- * $Id$ 
- *
  * POP3 service for the Citadel system
  *
  * Copyright (c) 1998-2009 by the citadel.org team
@@ -83,7 +81,7 @@ void pop3_cleanup_function(void) {
 	/* Don't do this stuff if this is not a POP3 session! */
 	if (CC->h_command_function != pop3_command_loop) return;
 
-	CtdlLogPrintf(CTDL_DEBUG, "Performing POP3 cleanup hook\n");
+	syslog(LOG_DEBUG, "Performing POP3 cleanup hook");
 	if (POP3->msgs != NULL) free(POP3->msgs);
 
 	free(POP3);
@@ -100,8 +98,7 @@ void pop3_greeting(void) {
 	CC->session_specific_data = malloc(sizeof(struct citpop3));
 	memset(POP3, 0, sizeof(struct citpop3));
 
-	cprintf("+OK Citadel POP3 server %s\r\n",
-		CC->cs_nonce);
+	cprintf("+OK Citadel POP3 server ready.\r\n");
 }
 
 
@@ -113,9 +110,9 @@ void pop3s_greeting(void) {
 
 /* kill session if no crypto */
 #ifdef HAVE_OPENSSL
-	if (!CC->redirect_ssl) CC->kill_me = 1;
+	if (!CC->redirect_ssl) CC->kill_me = KILLME_NO_CRYPTO;
 #else
-	CC->kill_me = 1;
+	CC->kill_me = KILLME_NO_CRYPTO;
 #endif
 
 	pop3_greeting();
@@ -137,7 +134,7 @@ void pop3_user(char *argbuf) {
 	strcpy(username, argbuf);
 	striplt(username);
 
-	/* CtdlLogPrintf(CTDL_DEBUG, "Trying <%s>\n", username); */
+	/* syslog(LOG_DEBUG, "Trying <%s>", username); */
 	if (CtdlLoginExistingUser(NULL, username) == login_ok) {
 		cprintf("+OK Password required for %s\r\n", username);
 	}
@@ -215,7 +212,7 @@ void pop3_login(void)
 	if (msgs >= 0) {
 		cprintf("+OK %s is logged in (%d messages)\r\n",
 			CC->user.fullname, msgs);
-		CtdlLogPrintf(CTDL_NOTICE, "POP3 authenticated %s\n", CC->user.fullname);
+		syslog(LOG_NOTICE, "POP3 authenticated %s", CC->user.fullname);
 	}
 	else {
 		cprintf("-ERR Can't open your mailbox\r\n");
@@ -223,60 +220,6 @@ void pop3_login(void)
 	
 }
 
-void pop3_apop(char *argbuf)
-{
-   char username[SIZ];
-   char userdigest[MD5_HEXSTRING_SIZE];
-   char realdigest[MD5_HEXSTRING_SIZE];
-   char *sptr;
-   
-   if (CC->logged_in)
-   {
-   	cprintf("-ERR You are already logged in; not in the AUTHORIZATION phase.\r\n");
-   	return;
-   }
-   
-   if ((sptr = strchr(argbuf, ' ')) == NULL)
-   {
-   	cprintf("-ERR Invalid APOP line.\r\n");
-   	return;
-   }
-   
-   *sptr++ = '\0';
-   
-   while ((*sptr) && isspace(*sptr))
-      sptr++;
-   
-   strncpy(username, argbuf, sizeof(username)-1);
-   username[sizeof(username)-1] = '\0';
-   
-   memset(userdigest, 0, MD5_HEXSTRING_SIZE);
-   strncpy(userdigest, sptr, MD5_HEXSTRING_SIZE-1);
-   
-   if (CtdlLoginExistingUser(NULL, username) != login_ok)
-   {
-   	cprintf("-ERR No such user.\r\n");
-   	return;
-   }
-   
-   if (CtdlGetUser(&CC->user, CC->curr_user))
-   {
-   	cprintf("-ERR No such user.\r\n");
-   	return;
-   }
-   
-   make_apop_string(CC->user.password, CC->cs_nonce, realdigest, sizeof realdigest);
-   if (!strncasecmp(realdigest, userdigest, MD5_HEXSTRING_SIZE-1))
-   {
-	do_login();
-   	pop3_login();
-   }
-   else
-   {
-	cprintf("-ERR That is NOT the password.\r\n");
-   }
-}
-
 
 /*
  * Authorize with password (implements POP3 "PASS" command)
@@ -287,7 +230,7 @@ void pop3_pass(char *argbuf) {
 	safestrncpy(password, argbuf, sizeof password);
 	striplt(password);
 
-	/* CtdlLogPrintf(CTDL_DEBUG, "Trying <%s>\n", password); */
+	/* syslog(LOG_DEBUG, "Trying <%s>", password); */
 	if (CtdlTryPassword(password, strlen(password)) == pass_ok) {
 		pop3_login();
 	}
@@ -626,15 +569,15 @@ void pop3_command_loop(void) {
 	time(&CC->lastcmd);
 	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
 	if (client_getln(cmdbuf, sizeof cmdbuf) < 1) {
-		CtdlLogPrintf(CTDL_ERR, "Client disconnected: ending session.\r\n");
-		CC->kill_me = 1;
+		syslog(LOG_ERR, "POP3 client disconnected: ending session.");
+		CC->kill_me = KILLME_CLIENT_DISCONNECTED;
 		return;
 	}
 	if (!strncasecmp(cmdbuf, "PASS", 4)) {
-		CtdlLogPrintf(CTDL_INFO, "POP3: PASS...\r\n");
+		syslog(LOG_INFO, "POP3: PASS...");
 	}
 	else {
-		CtdlLogPrintf(CTDL_INFO, "POP3: %s\r\n", cmdbuf);
+		syslog(LOG_INFO, "POP3: %s", cmdbuf);
 	}
 	while (strlen(cmdbuf) < 5) strcat(cmdbuf, " ");
 
@@ -649,7 +592,7 @@ void pop3_command_loop(void) {
 	else if (!strncasecmp(cmdbuf, "QUIT", 4)) {
 		cprintf("+OK Goodbye...\r\n");
 		pop3_update();
-		CC->kill_me = 1;
+		CC->kill_me = KILLME_CLIENT_LOGGED_OUT;
 		return;
 	}
 
@@ -661,11 +604,6 @@ void pop3_command_loop(void) {
 		pop3_pass(&cmdbuf[5]);
 	}
 
-	else if (!strncasecmp(cmdbuf, "APOP", 4))
-	{
-		pop3_apop(&cmdbuf[5]);
-	}
-
 #ifdef HAVE_OPENSSL
 	else if (!strncasecmp(cmdbuf, "STLS", 4)) {
 		pop3_stls();
@@ -678,7 +616,7 @@ void pop3_command_loop(void) {
 	
 	else if (CC->nologin) {
 		cprintf("-ERR System busy, try later.\r\n");
-		CC->kill_me = 1;
+		CC->kill_me = KILLME_NOLOGIN;
 	}
 
 	else if (!strncasecmp(cmdbuf, "LIST", 4)) {
@@ -744,6 +682,6 @@ CTDL_MODULE_INIT(pop3)
 		CtdlRegisterSessionHook(pop3_cleanup_function, EVT_STOP);
 	}
 	
-	/* return our Subversion id for the Log */
-	return "$Id$";
+	/* return our module name for the log */
+	return "pop3";
 }
