@@ -1,6 +1,4 @@
 /*
- * $Id$
- *
  * This module glues libSieve to the Citadel server in order to implement
  * the Sieve mailbox filtering language (RFC 3028).
  *
@@ -66,7 +64,7 @@ char *msiv_extensions = NULL;
  */
 int ctdl_debug(sieve2_context_t *s, void *my)
 {
-	CtdlLogPrintf(CTDL_DEBUG, "Sieve: %s\n", sieve2_getvalue_string(s, "message"));
+	syslog(LOG_DEBUG, "Sieve: %s\n", sieve2_getvalue_string(s, "message"));
 	return SIEVE2_OK;
 }
 
@@ -76,7 +74,7 @@ int ctdl_debug(sieve2_context_t *s, void *my)
  */
 int ctdl_errparse(sieve2_context_t *s, void *my)
 {
-	CtdlLogPrintf(CTDL_WARNING, "Error in script, line %d: %s\n",
+	syslog(LOG_WARNING, "Error in script, line %d: %s\n",
 		sieve2_getvalue_int(s, "lineno"),
 		sieve2_getvalue_string(s, "message")
 	);
@@ -89,7 +87,7 @@ int ctdl_errparse(sieve2_context_t *s, void *my)
  */
 int ctdl_errexec(sieve2_context_t *s, void *my)
 {
-	CtdlLogPrintf(CTDL_WARNING, "Error executing script: %s\n",
+	syslog(LOG_WARNING, "Error executing script: %s\n",
 		sieve2_getvalue_string(s, "message")
 	);
 	return SIEVE2_OK;
@@ -108,22 +106,22 @@ int ctdl_redirect(sieve2_context_t *s, void *my)
 
 	safestrncpy(recp, sieve2_getvalue_string(s, "address"), sizeof recp);
 
-	CtdlLogPrintf(CTDL_DEBUG, "Action is REDIRECT, recipient <%s>\n", recp);
+	syslog(LOG_DEBUG, "Action is REDIRECT, recipient <%s>\n", recp);
 
 	valid = validate_recipients(recp, NULL, 0);
 	if (valid == NULL) {
-		CtdlLogPrintf(CTDL_WARNING, "REDIRECT failed: bad recipient <%s>\n", recp);
+		syslog(LOG_WARNING, "REDIRECT failed: bad recipient <%s>\n", recp);
 		return SIEVE2_ERROR_BADARGS;
 	}
 	if (valid->num_error > 0) {
-		CtdlLogPrintf(CTDL_WARNING, "REDIRECT failed: bad recipient <%s>\n", recp);
+		syslog(LOG_WARNING, "REDIRECT failed: bad recipient <%s>\n", recp);
 		free_recipients(valid);
 		return SIEVE2_ERROR_BADARGS;
 	}
 
 	msg = CtdlFetchMessage(cs->msgnum, 1);
 	if (msg == NULL) {
-		CtdlLogPrintf(CTDL_WARNING, "REDIRECT failed: unable to fetch msg %ld\n", cs->msgnum);
+		syslog(LOG_WARNING, "REDIRECT failed: unable to fetch msg %ld\n", cs->msgnum);
 		free_recipients(valid);
 		return SIEVE2_ERROR_BADARGS;
 	}
@@ -143,7 +141,7 @@ int ctdl_keep(sieve2_context_t *s, void *my)
 {
 	struct ctdl_sieve *cs = (struct ctdl_sieve *)my;
 	
-	CtdlLogPrintf(CTDL_DEBUG, "Action is KEEP\n");
+	syslog(LOG_DEBUG, "Action is KEEP\n");
 
 	cs->keep = 1;
 	cs->cancel_implicit_keep = 1;
@@ -162,7 +160,7 @@ int ctdl_fileinto(sieve2_context_t *s, void *my)
 	char foldername[256];
 	char original_room_name[ROOMNAMELEN];
 
-	CtdlLogPrintf(CTDL_DEBUG, "Action is FILEINTO, destination is <%s>\n", dest_folder);
+	syslog(LOG_DEBUG, "Action is FILEINTO, destination is <%s>\n", dest_folder);
 
 	/* FILEINTO 'INBOX' is the same thing as KEEP */
 	if ( (!strcasecmp(dest_folder, "INBOX")) || (!strcasecmp(dest_folder, MAILROOM)) ) {
@@ -185,7 +183,7 @@ int ctdl_fileinto(sieve2_context_t *s, void *my)
 	}
 
 	if (c != 0) {
-		CtdlLogPrintf(CTDL_WARNING, "FILEINTO failed: target <%s> does not exist\n", dest_folder);
+		syslog(LOG_WARNING, "FILEINTO failed: target <%s> does not exist\n", dest_folder);
 		return SIEVE2_ERROR_BADARGS;
 	}
 
@@ -216,7 +214,7 @@ int ctdl_discard(sieve2_context_t *s, void *my)
 {
 	struct ctdl_sieve *cs = (struct ctdl_sieve *)my;
 
-	CtdlLogPrintf(CTDL_DEBUG, "Action is DISCARD\n");
+	syslog(LOG_DEBUG, "Action is DISCARD\n");
 
 	/* Cancel the implicit keep.  That's all there is to it. */
 	cs->cancel_implicit_keep = 1;
@@ -233,11 +231,11 @@ int ctdl_reject(sieve2_context_t *s, void *my)
 	struct ctdl_sieve *cs = (struct ctdl_sieve *)my;
 	char *reject_text = NULL;
 
-	CtdlLogPrintf(CTDL_DEBUG, "Action is REJECT\n");
+	syslog(LOG_DEBUG, "Action is REJECT\n");
 
 	/* If we don't know who sent the message, do a DISCARD instead. */
 	if (IsEmptyStr(cs->sender)) {
-		CtdlLogPrintf(CTDL_INFO, "Unknown sender.  Doing DISCARD instead of REJECT.\n");
+		syslog(LOG_INFO, "Unknown sender.  Doing DISCARD instead of REJECT.\n");
 		return ctdl_discard(s, my);
 	}
 
@@ -288,7 +286,7 @@ int ctdl_vacation(sieve2_context_t *s, void *my)
 	char *vacamsg_text = NULL;
 	char vacamsg_subject[1024];
 
-	CtdlLogPrintf(CTDL_DEBUG, "Action is VACATION\n");
+	syslog(LOG_DEBUG, "Action is VACATION\n");
 
 	message = sieve2_getvalue_string(s, "message");
 	if (message == NULL) return SIEVE2_ERROR_BADARGS;
@@ -308,7 +306,7 @@ int ctdl_vacation(sieve2_context_t *s, void *my)
 	for (vptr = cs->u->first_vacation; vptr != NULL; vptr = vptr->next) {
 		if (!strcasecmp(vptr->fromaddr, cs->sender)) {
 			if ( (time(NULL) - vptr->timestamp) < (days * 86400) ) {
-				CtdlLogPrintf(CTDL_DEBUG, "Already alerted <%s> recently.\n", cs->sender);
+				syslog(LOG_DEBUG, "Already alerted <%s> recently.\n", cs->sender);
 				return SIEVE2_OK;
 			}
 		}
@@ -393,7 +391,7 @@ int ctdl_getenvelope(sieve2_context_t *s, void *my)
 {
 	struct ctdl_sieve *cs = (struct ctdl_sieve *)my;
 
-	CtdlLogPrintf(CTDL_DEBUG, "Action is GETENVELOPE\nEnvFrom: %s\n  EnvTo: %s\n",
+	syslog(LOG_DEBUG, "Action is GETENVELOPE\nEnvFrom: %s\n  EnvTo: %s\n",
 		cs->envelope_from, cs->envelope_to);
 
 	if (cs->envelope_from != NULL) {
@@ -465,13 +463,13 @@ int ctdl_getscript(sieve2_context_t *s, void *my) {
 
 	for (sptr=cs->u->first_script; sptr!=NULL; sptr=sptr->next) {
 		if (sptr->script_active > 0) {
-			CtdlLogPrintf(CTDL_DEBUG, "ctdl_getscript() is using script '%s'\n", sptr->script_name);
+			syslog(LOG_DEBUG, "ctdl_getscript() is using script '%s'\n", sptr->script_name);
 			sieve2_setvalue_string(s, "script", sptr->script_content);
 			return SIEVE2_OK;
 		}
 	}
 		
-	CtdlLogPrintf(CTDL_DEBUG, "ctdl_getscript() found no active script\n");
+	syslog(LOG_DEBUG, "ctdl_getscript() found no active script\n");
 	return SIEVE2_ERROR_GETSCRIPT;
 }
 
@@ -482,7 +480,7 @@ int ctdl_getheaders(sieve2_context_t *s, void *my) {
 
 	struct ctdl_sieve *cs = (struct ctdl_sieve *)my;
 
-	CtdlLogPrintf(CTDL_DEBUG, "ctdl_getheaders() was called\n");
+	syslog(LOG_DEBUG, "ctdl_getheaders() was called\n");
 	sieve2_setvalue_string(s, "allheaders", cs->rfc822headers);
 	return SIEVE2_OK;
 }
@@ -503,7 +501,7 @@ void sieve_queue_room(struct ctdlroom *which_room) {
 	ptr->next = sieve_list;
 	sieve_list = ptr;
 	end_critical_section(S_SIEVELIST);
-	CtdlLogPrintf(CTDL_DEBUG, "<%s> queued for Sieve processing\n", which_room->QRname);
+	syslog(LOG_DEBUG, "<%s> queued for Sieve processing\n", which_room->QRname);
 }
 
 
@@ -523,13 +521,13 @@ void sieve_do_msg(long msgnum, void *userdata) {
 
 	if (u == NULL)
 	{
-		CtdlLogPrintf(CTDL_EMERG, "Can't process message <%ld> without userdata!\n", msgnum);
+		syslog(LOG_EMERG, "Can't process message <%ld> without userdata!\n", msgnum);
 		return;
 	}
 
 	sieve2_context = u->sieve2_context;
 
-	CtdlLogPrintf(CTDL_DEBUG, "Performing sieve processing on msg <%ld>\n", msgnum);
+	syslog(LOG_DEBUG, "Performing sieve processing on msg <%ld>\n", msgnum);
 
 	/*
 	 * Make sure you include message body so you can get those second-level headers ;)
@@ -640,10 +638,10 @@ void sieve_do_msg(long msgnum, void *userdata) {
 
 	sieve2_setvalue_string(sieve2_context, "allheaders", my.rfc822headers);
 	
-	CtdlLogPrintf(CTDL_DEBUG, "Calling sieve2_execute()\n");
+	syslog(LOG_DEBUG, "Calling sieve2_execute()\n");
 	res = sieve2_execute(sieve2_context, &my);
 	if (res != SIEVE2_OK) {
-		CtdlLogPrintf(CTDL_CRIT, "sieve2_execute() returned %d: %s\n", res, sieve2_errstr(res));
+		syslog(LOG_CRIT, "sieve2_execute() returned %d: %s\n", res, sieve2_errstr(res));
 	}
 
 	free(my.rfc822headers);
@@ -654,11 +652,11 @@ void sieve_do_msg(long msgnum, void *userdata) {
 	 * if no other action was successfully taken.
 	 */
 	if ( (!my.keep) && (my.cancel_implicit_keep) ) {
-		CtdlLogPrintf(CTDL_DEBUG, "keep is 0 -- deleting message from inbox\n");
+		syslog(LOG_DEBUG, "keep is 0 -- deleting message from inbox\n");
 		CtdlDeleteMessages(CC->room.QRname, &msgnum, 1, "");
 	}
 
-	CtdlLogPrintf(CTDL_DEBUG, "Completed sieve processing on msg <%ld>\n", msgnum);
+	syslog(LOG_DEBUG, "Completed sieve processing on msg <%ld>\n", msgnum);
 	u->lastproc = msgnum;
 
 	return;
@@ -867,7 +865,7 @@ void sieve_do_room(char *roomname) {
 	 */
 	snprintf(u.config_roomname, sizeof u.config_roomname, "%010ld.%s", atol(roomname), USERCONFIGROOM);
 	if (CtdlGetRoom(&CC->room, u.config_roomname) != 0) {
-		CtdlLogPrintf(CTDL_DEBUG, "<%s> does not exist.  No processing is required.\n", u.config_roomname);
+		syslog(LOG_DEBUG, "<%s> does not exist.  No processing is required.\n", u.config_roomname);
 		return;
 	}
 
@@ -879,14 +877,14 @@ void sieve_do_room(char *roomname) {
 		get_sieve_config_backend, (void *)&u );
 
 	if (u.config_msgnum < 0) {
-		CtdlLogPrintf(CTDL_DEBUG, "No Sieve rules exist.  No processing is required.\n");
+		syslog(LOG_DEBUG, "No Sieve rules exist.  No processing is required.\n");
 		return;
 	}
 
-	CtdlLogPrintf(CTDL_DEBUG, "Rules found.  Performing Sieve processing for <%s>\n", roomname);
+	syslog(LOG_DEBUG, "Rules found.  Performing Sieve processing for <%s>\n", roomname);
 
 	if (CtdlGetRoom(&CC->room, roomname) != 0) {
-		CtdlLogPrintf(CTDL_CRIT, "ERROR: cannot load <%s>\n", roomname);
+		syslog(LOG_CRIT, "ERROR: cannot load <%s>\n", roomname);
 		return;
 	}
 
@@ -894,13 +892,13 @@ void sieve_do_room(char *roomname) {
 	
 	res = sieve2_alloc(&sieve2_context);
 	if (res != SIEVE2_OK) {
-		CtdlLogPrintf(CTDL_CRIT, "sieve2_alloc() returned %d: %s\n", res, sieve2_errstr(res));
+		syslog(LOG_CRIT, "sieve2_alloc() returned %d: %s\n", res, sieve2_errstr(res));
 		return;
 	}
 
 	res = sieve2_callbacks(sieve2_context, ctdl_sieve_callbacks);
 	if (res != SIEVE2_OK) {
-		CtdlLogPrintf(CTDL_CRIT, "sieve2_callbacks() returned %d: %s\n", res, sieve2_errstr(res));
+		syslog(LOG_CRIT, "sieve2_callbacks() returned %d: %s\n", res, sieve2_errstr(res));
 		goto BAIL;
 	}
 
@@ -911,7 +909,7 @@ void sieve_do_room(char *roomname) {
 	my.u = &u;
 	res = sieve2_validate(sieve2_context, &my);
 	if (res != SIEVE2_OK) {
-		CtdlLogPrintf(CTDL_CRIT, "sieve2_validate() returned %d: %s\n", res, sieve2_errstr(res));
+		syslog(LOG_CRIT, "sieve2_validate() returned %d: %s\n", res, sieve2_errstr(res));
 		goto BAIL;
 	}
 
@@ -926,7 +924,7 @@ void sieve_do_room(char *roomname) {
 BAIL:
 	res = sieve2_free(&sieve2_context);
 	if (res != SIEVE2_OK) {
-		CtdlLogPrintf(CTDL_CRIT, "sieve2_free() returned %d: %s\n", res, sieve2_errstr(res));
+		syslog(LOG_CRIT, "sieve2_free() returned %d: %s\n", res, sieve2_errstr(res));
 	}
 
 	/* Rewrite the config if we have to */
@@ -941,7 +939,7 @@ void perform_sieve_processing(void) {
 	struct RoomProcList *ptr = NULL;
 
 	if (sieve_list != NULL) {
-		CtdlLogPrintf(CTDL_DEBUG, "Begin Sieve processing\n");
+		syslog(LOG_DEBUG, "Begin Sieve processing\n");
 		while (sieve_list != NULL) {
 			char spoolroomname[ROOMNAMELEN];
 			safestrncpy(spoolroomname, sieve_list->name, sizeof spoolroomname);
@@ -1258,7 +1256,7 @@ void ctdl_sieve_init(void) {
 		strcpy(&cred[55], "...");
 	}
 
-	CtdlLogPrintf(CTDL_INFO, "%s\n",cred);
+	syslog(LOG_INFO, "%s\n",cred);
 	free(cred);
 
 	/* Briefly initialize a Sieve parser instance just so we can list the
@@ -1266,22 +1264,22 @@ void ctdl_sieve_init(void) {
 	 */
 	res = sieve2_alloc(&sieve2_context);
 	if (res != SIEVE2_OK) {
-		CtdlLogPrintf(CTDL_CRIT, "sieve2_alloc() returned %d: %s\n", res, sieve2_errstr(res));
+		syslog(LOG_CRIT, "sieve2_alloc() returned %d: %s\n", res, sieve2_errstr(res));
 		return;
 	}
 
 	res = sieve2_callbacks(sieve2_context, ctdl_sieve_callbacks);
 	if (res != SIEVE2_OK) {
-		CtdlLogPrintf(CTDL_CRIT, "sieve2_callbacks() returned %d: %s\n", res, sieve2_errstr(res));
+		syslog(LOG_CRIT, "sieve2_callbacks() returned %d: %s\n", res, sieve2_errstr(res));
 		goto BAIL;
 	}
 
 	msiv_extensions = strdup(sieve2_listextensions(sieve2_context));
-	CtdlLogPrintf(CTDL_INFO, "Extensions: %s\n", msiv_extensions);
+	syslog(LOG_INFO, "Extensions: %s\n", msiv_extensions);
 
 BAIL:	res = sieve2_free(&sieve2_context);
 	if (res != SIEVE2_OK) {
-		CtdlLogPrintf(CTDL_CRIT, "sieve2_free() returned %d: %s\n", res, sieve2_errstr(res));
+		syslog(LOG_CRIT, "sieve2_free() returned %d: %s\n", res, sieve2_errstr(res));
 	}
 
 }
@@ -1325,7 +1323,7 @@ CTDL_MODULE_INIT(sieve)
 		CtdlRegisterCleanupHook(cleanup_sieve);
 	}
 	
-        /* return our Subversion id for the Log */
-	return "$Id$";
+        /* return our module name for the log */
+	return "sieve";
 }
 
