@@ -1,24 +1,21 @@
 /*
- * $Id$
- *
  * A skeleton module to test the dynamic loader.
  *
- *
  * Copyright (c) 1998-2009 by the citadel.org team
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 #include "sysdep.h"
@@ -49,25 +46,25 @@
 
 
 void CleanupTest(void) {
-	CtdlLogPrintf(CTDL_DEBUG, "--- test of adding an unload hook --- \n");
+	syslog(LOG_DEBUG, "--- test of adding an unload hook --- \n");
 	}
 
 void NewRoomTest(void) {
-	CtdlLogPrintf(CTDL_DEBUG, "--- test module was told we're now in a new room ---\n");
+	syslog(LOG_DEBUG, "--- test module was told we're now in a new room ---\n");
 	}
 
 void SessionStartTest(void) {
-	CtdlLogPrintf(CTDL_DEBUG, "--- starting up session %d ---\n",
+	syslog(LOG_DEBUG, "--- starting up session %d ---\n",
 		CC->cs_pid);
 	}
 
 void SessionStopTest(void) {
-	CtdlLogPrintf(CTDL_DEBUG, "--- ending session %d ---\n", 
+	syslog(LOG_DEBUG, "--- ending session %d ---\n", 
 		CC->cs_pid);
 	}
 
 void LoginTest(void) {
-	CtdlLogPrintf(CTDL_DEBUG, "--- Hello, %s ---\n", CC->curr_user);
+	syslog(LOG_DEBUG, "--- Hello, %s ---\n", CC->curr_user);
 	}
 
 /* To insert this module into the server activate the next block by changing the #if 0 to #if 1 */
@@ -84,6 +81,6 @@ CTDL_MODULE_INIT(test)
 	}
 #endif
 
-   /* return our Subversion id for the Log */
-   return "$Id$";
+   /* return our module name for the log */
+   return "test";
 }
