@@ -1,24 +1,22 @@
 /*
- * $Id$
- * 
  * A server-side module for Citadel which supports address book information
  * using the standard vCard format.
  * 
  * Copyright (c) 1999-2009 by the citadel.org team
  *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 3 of the License, or
- *  (at your option) any later version.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
 /*
@@ -148,18 +146,18 @@ void vcard_directory_add_user(char *internet_addr, char *citadel_addr) {
 	 * probably just the networker or something.
 	 */
 	if (CC->logged_in) {
-		CtdlLogPrintf(CTDL_DEBUG, "Checking for <%s>...\n", internet_addr);
+		syslog(LOG_DEBUG, "Checking for <%s>...\n", internet_addr);
 		if (CtdlDirectoryLookup(buf, internet_addr, sizeof buf) == 0) {
 			if (strcasecmp(buf, citadel_addr)) {
 				/* This address belongs to someone else.
 				 * Bail out silently without saving.
 				 */
-				CtdlLogPrintf(CTDL_DEBUG, "DOOP!\n");
+				syslog(LOG_DEBUG, "DOOP!\n");
 				return;
 			}
 		}
 	}
-	CtdlLogPrintf(CTDL_INFO, "Adding %s (%s) to directory\n", citadel_addr, internet_addr);
+	syslog(LOG_INFO, "Adding %s (%s) to directory\n", citadel_addr, internet_addr);
 	CtdlDirectoryAddUser(internet_addr, citadel_addr);
 }
 
@@ -200,7 +198,7 @@ void cmd_igab(char *argbuf) {
 	CtdlDirectoryInit();
 
 	/* We want *all* vCards in this room */
-	CtdlForEachMessage(MSGS_ALL, 0, NULL, "^[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$",
+	CtdlForEachMessage(MSGS_ALL, 0, NULL, "[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$",
 		NULL, vcard_add_to_directory, NULL);
 
 	CtdlGetRoom(&CC->room, hold_rm);	/* return to saved room */
@@ -286,7 +284,7 @@ void vcard_extract_vcard(char *name, char *filename, char *partnum, char *disp,
 	if (  (!strcasecmp(cbtype, "text/x-vcard"))
 	   || (!strcasecmp(cbtype, "text/vcard")) ) {
 
-		CtdlLogPrintf(CTDL_DEBUG, "Part %s contains a vCard!  Loading...\n", partnum);
+		syslog(LOG_DEBUG, "Part %s contains a vCard!  Loading...\n", partnum);
 		if (*v != NULL) {
 			vcard_free(*v);
 		}
@@ -364,7 +362,6 @@ int vcard_upload_beforesave(struct CtdlMessage *msg) {
 	}
 
 	s = vcard_get_prop(v, "fn", 1, 0, 0);
-	if (s) CtdlLogPrintf(CTDL_DEBUG, "vCard beforesave hook running for <%s>\n", s);
 
 	if (yes_my_citadel_config) {
 		/* Bingo!  The user is uploading a new vCard, so
@@ -395,7 +392,7 @@ int vcard_upload_beforesave(struct CtdlMessage *msg) {
 		 * vCard in the user's config room at all times.
 		 *
 		 */
-		CtdlDeleteMessages(CC->room.QRname, NULL, 0, "^[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$");
+		CtdlDeleteMessages(CC->room.QRname, NULL, 0, "[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$");
 
 		/* Make the author of the message the name of the user. */
 		if (msg->cm_fields['A'] != NULL) {
@@ -524,8 +521,8 @@ int vcard_upload_aftersave(struct CtdlMessage *msg) {
 			 * copy it to the Global Address Book room.
 			 */
 
-			I = atol(msg->cm_fields['I']);
-			if (I < 0L) return(0);
+			I = atol(msg->cm_fields['3']);
+			if (I <= 0L) return(0);
 
 			/* Store our Internet return address in memory */
 			if (is_MY_UserConf) {
@@ -609,7 +606,7 @@ struct vCard *vcard_get_user(struct ctdluser *u) {
 
 	/* We want the last (and probably only) vcard in this room */
 	VCmsgnum = (-1);
-	CtdlForEachMessage(MSGS_LAST, 1, NULL, "^[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$",
+	CtdlForEachMessage(MSGS_LAST, 1, NULL, "[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$",
 		NULL, vcard_gu_backend, (void *)&VCmsgnum );
 	CtdlGetRoom(&CC->room, hold_rm);	/* return to saved room */
 
@@ -812,7 +809,7 @@ void vcard_newuser(struct ctdluser *usbuf) {
 	struct vCard *v;
 
 	vcard_fn_to_n(vname, usbuf->fullname, sizeof vname);
-	CtdlLogPrintf(CTDL_DEBUG, "Converted <%s> to <%s>\n", usbuf->fullname, vname);
+	syslog(LOG_DEBUG, "Converted <%s> to <%s>\n", usbuf->fullname, vname);
 
 	/* Create and save the vCard */
 	v = vcard_new();
@@ -831,7 +828,7 @@ void vcard_newuser(struct ctdluser *usbuf) {
 		if (getpwuid_r(usbuf->uid, &pwd, pwd_buffer, sizeof pwd_buffer) != NULL) {
 #else // SOLARIS_GETPWUID
 		struct passwd *result = NULL;
-		CtdlLogPrintf(CTDL_DEBUG, "Searching for uid %d\n", usbuf->uid);
+		syslog(LOG_DEBUG, "Searching for uid %d\n", usbuf->uid);
 		if (getpwuid_r(usbuf->uid, &pwd, pwd_buffer, sizeof pwd_buffer, &result) == 0) {
 #endif // HAVE_GETPWUID_R
 			snprintf(buf, sizeof buf, "%s@%s", pwd.pw_name, config.c_fqdn);
@@ -1118,13 +1115,13 @@ void check_get(void) {
 	time(&CC->lastcmd);
 	memset(cmdbuf, 0, sizeof cmdbuf); /* Clear it, just in case */
 	if (client_getln(cmdbuf, sizeof cmdbuf) < 1) {
-		CtdlLogPrintf(CTDL_CRIT, "Client disconnected: ending session.\n");
-		CC->kill_me = 1;
+		syslog(LOG_CRIT, "vcard client disconnected: ending session.\n");
+		CC->kill_me = KILLME_CLIENT_DISCONNECTED;
 		return;
 	}
-	CtdlLogPrintf(CTDL_INFO, ": %s\n", cmdbuf);
+	syslog(LOG_INFO, ": %s\n", cmdbuf);
 	while (strlen(cmdbuf) < 3) strcat(cmdbuf, " ");
-	CtdlLogPrintf(CTDL_INFO, "[ %s]\n", cmdbuf);
+	syslog(LOG_INFO, "[ %s]\n", cmdbuf);
 	
 	if (strncasecmp(cmdbuf, "GET ", 4)==0)
 	{
@@ -1141,13 +1138,13 @@ void check_get(void) {
 		{
 
 			cprintf("200 OK %s\n", internet_addr);
-			CtdlLogPrintf(CTDL_INFO, "sending 200 OK for the room %s\n", rcpt->display_recp);
+			syslog(LOG_INFO, "sending 200 OK for the room %s\n", rcpt->display_recp);
 		}
 		else 
 		{
 			cprintf("500 REJECT noone here by that name.\n");
 			
-			CtdlLogPrintf(CTDL_INFO, "sending 500 REJECT noone here by that name: %s\n", internet_addr);
+			syslog(LOG_INFO, "sending 500 REJECT noone here by that name: %s\n", internet_addr);
 		}
 		if (rcpt != NULL) 
 			free_recipients(rcpt);
@@ -1156,7 +1153,7 @@ void check_get(void) {
 	{
 		cprintf("500 REJECT invalid Query.\n");
 		
-		CtdlLogPrintf(CTDL_INFO, "sending 500 REJECT invalid Query: %s\n", internet_addr);
+		syslog(LOG_INFO, "sending 500 REJECT invalid Query: %s\n", internet_addr);
 	}
 }
 
@@ -1178,7 +1175,7 @@ void vcard_CtdlCreateRoom(void)
 
 	/* Set expiration policy to manual; otherwise objects will be lost! */
 	if (CtdlGetRoomLock(&qr, USERCONTACTSROOM)) {
-		CtdlLogPrintf(CTDL_ERR, "Couldn't get the user CONTACTS room!\n");
+		syslog(LOG_ERR, "Couldn't get the user CONTACTS room!\n");
 		return;
 	}
 	qr.QRep.expire_mode = EXPIRE_MANUAL;
@@ -1317,7 +1314,6 @@ void strip_addresses_already_have(long msgnum, void *userdata) {
  */
 void store_this_ha(struct addresses_to_be_filed *aptr) {
 	struct CtdlMessage *vmsg = NULL;
-	long vmsgnum = (-1L);
 	char *ser = NULL;
 	struct vCard *v = NULL;
 	char recipient[256];
@@ -1325,7 +1321,7 @@ void store_this_ha(struct addresses_to_be_filed *aptr) {
 
 	/* First remove any addresses we already have in the address book */
 	CtdlUserGoto(aptr->roomname, 0, 0, NULL, NULL);
-	CtdlForEachMessage(MSGS_ALL, 0, NULL, "^[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$", NULL,
+	CtdlForEachMessage(MSGS_ALL, 0, NULL, "[Tt][Ee][Xx][Tt]/.*[Vv][Cc][Aa][Rr][Dd]$", NULL,
 		strip_addresses_already_have, aptr->collected_addresses);
 
 	if (!IsEmptyStr(aptr->collected_addresses))
@@ -1353,8 +1349,8 @@ void store_this_ha(struct addresses_to_be_filed *aptr) {
 			}
 			vcard_free(v);
 
-			CtdlLogPrintf(CTDL_DEBUG, "Adding contact: %s\n", recipient);
-			vmsgnum = CtdlSubmitMsg(vmsg, NULL, aptr->roomname, QP_EADDR);
+			syslog(LOG_DEBUG, "Adding contact: %s\n", recipient);
+			CtdlSubmitMsg(vmsg, NULL, aptr->roomname, QP_EADDR);
 			CtdlFreeMessage(vmsg);
 		}
 	}
@@ -1461,6 +1457,9 @@ CTDL_MODULE_INIT(vcard)
 			fp = fopen(filename, "a");
 			if (fp != NULL) fclose(fp);
 			rv = chown(filename, CTDLUID, (-1));
+			if (rv == -1)
+				syslog(LOG_EMERG, "Failed to adjust ownership of: %s [%s]\n", 
+				       filename, strerror(errno));
 		}
 
 		/* for postfix tcpdict */
@@ -1472,6 +1471,6 @@ CTDL_MODULE_INIT(vcard)
 					CitadelServiceDICT_TCP);
 	}
 	
-	/* return our Subversion id for the Log */
-	return "$Id$";
+	/* return our module name for the log */
+	return "vcard";
 }
