@@ -1,4 +1,3 @@
-/* $Id$ */
 
 #ifndef MSGBASE_H
 #define MSGBASE_H
@@ -29,7 +28,8 @@ enum {
 	om_ok,
 	om_not_logged_in,
 	om_no_such_msg,
-	om_mime_error
+	om_mime_error,
+	om_access_denied
 };
 
 /*
@@ -49,6 +49,7 @@ struct ma_info {
 	char chosen_part[128];	/* Which part of a m/a did we choose? */
 	int chosen_pref;	/* Chosen part preference level (lower is better) */
 	int use_fo_hooks;	/* Use fixed output hooks */
+	int dont_decode;        /* should we call the decoder or not? */
 };
 
 
@@ -187,10 +188,13 @@ enum {
 	CHECK_EXISTANCE,
 	POST_LMTP
 };
+
 int CtdlDoIHavePermissionToPostInThisRoom(char *errmsgbuf, 
-					  size_t n, 
-					  const char* RemoteIdentifier,
-					  int PostPublic);
+	size_t n, 
+	const char* RemoteIdentifier,
+	int PostPublic,
+	int is_reply
+);
 
 
 /* values for which_set */
