@@ -1,8 +1,21 @@
 /* 
- * $Id$
- * 
  * Server functions which perform operations on room objects.
  *
+ * Copyright (c) 1987-2011 by the citadel.org team
+ *
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "sysdep.h"
@@ -54,21 +67,43 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 {
 	int retval = 0;
 	visit vbuf;
+	int is_me = 0;
+	int is_guest = 0;
+
+	if (userbuf == &CC->user) {
+		is_me = 1;
+	}
+
+	if ((is_me) && (config.c_guest_logins) && (!CC->logged_in)) {
+		is_guest = 1;
+	}
 
 	/* for internal programs, always do everything */
 	if (((CC->internal_pgm)) && (roombuf->QRflags & QR_INUSE)) {
-		retval = (UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_DELETEALLOWED);
+		retval = (UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_DELETEALLOWED | UA_REPLYALLOWED);
+		vbuf.v_view = 0;
+		goto SKIP_EVERYTHING;
+	}
+
+	/* If guest mode is enabled, always grant access to the Lobby */
+	if ((is_guest) && (!strcasecmp(roombuf->QRname, BASEROOM))) {
+		retval = (UA_KNOWN | UA_GOTOALLOWED);
 		vbuf.v_view = 0;
 		goto SKIP_EVERYTHING;
 	}
 
 	/* Locate any applicable user/room relationships */
-	CtdlGetRelationship(&vbuf, userbuf, roombuf);
+	if (is_guest) {
+		memset(&vbuf, 0, sizeof vbuf);
+	}
+	else {
+		CtdlGetRelationship(&vbuf, userbuf, roombuf);
+	}
 
 	/* Force the properties of the Aide room */
 	if (!strcasecmp(roombuf->QRname, config.c_aideroom)) {
 		if (userbuf->axlevel >= AxAideU) {
-			retval = UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_DELETEALLOWED;
+			retval = UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_DELETEALLOWED | UA_REPLYALLOWED;
 		} else {
 			retval = 0;
 		}
@@ -76,8 +111,9 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 	}
 
 	/* If this is a public room, it's accessible... */
-	if ( ((roombuf->QRflags & QR_PRIVATE) == 0) 
-	   && ((roombuf->QRflags & QR_MAILBOX) == 0) ) {
+	if (	((roombuf->QRflags & QR_PRIVATE) == 0) 
+		&& ((roombuf->QRflags & QR_MAILBOX) == 0)
+	) {
 		retval = retval | UA_KNOWN | UA_GOTOALLOWED;
 	}
 
@@ -89,8 +125,9 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 	}
 
 	/* For private rooms, check the generation number matchups */
-	if ( (roombuf->QRflags & QR_PRIVATE) 
-	   && ((roombuf->QRflags & QR_MAILBOX) == 0) ) {
+	if (	(roombuf->QRflags & QR_PRIVATE) 
+		&& ((roombuf->QRflags & QR_MAILBOX) == 0)
+	) {
 
 		/* An explicit match means the user belongs in this room */
 		if (vbuf.v_flags & V_ACCESS) {
@@ -99,8 +136,9 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 		/* Otherwise, check if this is a guess-name or passworded
 		 * room.  If it is, a goto may at least be attempted
 		 */
-		else if ((roombuf->QRflags & QR_PRIVATE)
-			 || (roombuf->QRflags & QR_PASSWORDED)) {
+		else if (	(roombuf->QRflags & QR_PRIVATE)
+				|| (roombuf->QRflags & QR_PASSWORDED)
+		) {
 			retval = retval & ~UA_KNOWN;
 			retval = retval | UA_GOTOALLOWED;
 		}
@@ -108,13 +146,13 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 
 	/* For mailbox rooms, also check the namespace */
 	/* Also, mailbox owners can delete their messages */
-	if (roombuf->QRflags & QR_MAILBOX) {
+	if ( (roombuf->QRflags & QR_MAILBOX) && (atol(roombuf->QRname) != 0)) {
 		if (userbuf->usernum == atol(roombuf->QRname)) {
-			retval = retval | UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_DELETEALLOWED;
+			retval = retval | UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_DELETEALLOWED | UA_REPLYALLOWED;
 		}
 		/* An explicit match means the user belongs in this room */
 		if (vbuf.v_flags & V_ACCESS) {
-			retval = retval | UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_DELETEALLOWED;
+			retval = retval | UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_DELETEALLOWED | UA_REPLYALLOWED;
 		}
 	}
 
@@ -125,19 +163,36 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 		 * - User is not validated
 		 * - User has no net privileges and it is a shared network room
 		 * - It is a read-only room
+		 * - It is a blog room (in which case we only allow replies to existing messages)
 		 */
 		int post_allowed = 1;
-		if (CC->user.axlevel < AxProbU) post_allowed = 0;
-		if ((CC->user.axlevel < AxNetU) && (CC->room.QRflags & QR_NETWORK)) post_allowed = 0;
-		if (roombuf->QRflags & QR_READONLY) post_allowed = 0;
+		int reply_allowed = 1;
+		if (userbuf->axlevel < AxProbU) {
+			post_allowed = 0;
+			reply_allowed = 0;
+		}
+		if ((userbuf->axlevel < AxNetU) && (roombuf->QRflags & QR_NETWORK)) {
+			post_allowed = 0;
+			reply_allowed = 0;
+		}
+		if (roombuf->QRflags & QR_READONLY) {
+			post_allowed = 0;
+			reply_allowed = 0;
+		}
+		if (roombuf->QRdefaultview == VIEW_BLOG) {
+			post_allowed = 0;
+		}
 		if (post_allowed) {
-			retval = retval | UA_POSTALLOWED;
+			retval = retval | UA_POSTALLOWED | UA_REPLYALLOWED;
+		}
+		if (reply_allowed) {
+			retval = retval | UA_REPLYALLOWED;
 		}
 
 		/* If "collaborative deletion" is active for this room, any user who can post
 		 * is also allowed to delete
 		 */
-		if (CC->room.QRflags2 & QR2_COLLABDEL) {
+		if (roombuf->QRflags2 & QR2_COLLABDEL) {
 			if (retval & UA_POSTALLOWED) {
 				retval = retval | UA_DELETEALLOWED;
 			}
@@ -148,42 +203,46 @@ void CtdlRoomAccess(struct ctdlroom *roombuf, struct ctdluser *userbuf,
 	/* Check to see if the user has forgotten this room */
 	if (vbuf.v_flags & V_FORGET) {
 		retval = retval & ~UA_KNOWN;
-		if ( ( ((roombuf->QRflags & QR_PRIVATE) == 0) 
-		      && ((roombuf->QRflags & QR_MAILBOX) == 0) )
-		   || ( (roombuf->QRflags & QR_MAILBOX) 
-		      && (atol(roombuf->QRname) == CC->user.usernum))) {
+		if (	( ((roombuf->QRflags & QR_PRIVATE) == 0) 
+			&& ((roombuf->QRflags & QR_MAILBOX) == 0)
+		) || (	(roombuf->QRflags & QR_MAILBOX) 
+			&& (atol(roombuf->QRname) == CC->user.usernum))
+		) {
 			retval = retval | UA_ZAPPED;
 		}
 	}
+
 	/* If user is explicitly locked out of this room, deny everything */
 	if (vbuf.v_flags & V_LOCKOUT) {
-		retval = retval & ~UA_KNOWN & ~UA_GOTOALLOWED & ~UA_POSTALLOWED;
+		retval = retval & ~UA_KNOWN & ~UA_GOTOALLOWED & ~UA_POSTALLOWED & ~UA_REPLYALLOWED;
 	}
 
 	/* Aides get access to all private rooms */
-	if ( (userbuf->axlevel >= AxAideU)
-	   && ((roombuf->QRflags & QR_MAILBOX) == 0) ) {
+	if (	(userbuf->axlevel >= AxAideU)
+		&& ((roombuf->QRflags & QR_MAILBOX) == 0)
+	) {
 		if (vbuf.v_flags & V_FORGET) {
-			retval = retval | UA_GOTOALLOWED | UA_POSTALLOWED;
+			retval = retval | UA_GOTOALLOWED | UA_POSTALLOWED | UA_REPLYALLOWED;
 		}
 		else {
-			retval = retval | UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED;
+			retval = retval | UA_KNOWN | UA_GOTOALLOWED | UA_POSTALLOWED | UA_REPLYALLOWED;
 		}
 	}
 
 	/* Aides can gain access to mailboxes as well, but they don't show
 	 * by default.
 	 */
-	if ( (userbuf->axlevel >= AxAideU)
-	   && (roombuf->QRflags & QR_MAILBOX) ) {
-		retval = retval | UA_GOTOALLOWED | UA_POSTALLOWED;
+	if (	(userbuf->axlevel >= AxAideU)
+		&& (roombuf->QRflags & QR_MAILBOX)
+	) {
+		retval = retval | UA_GOTOALLOWED | UA_POSTALLOWED | UA_REPLYALLOWED;
 	}
 
 	/* Aides and Room Aides have admin privileges */
-	if ( (userbuf->axlevel >= AxAideU)
-	   || (userbuf->usernum == roombuf->QRroomaide)
-	   ) {
-		retval = retval | UA_ADMINALLOWED | UA_DELETEALLOWED | UA_POSTALLOWED;
+	if (	(userbuf->axlevel >= AxAideU)
+		|| (userbuf->usernum == roombuf->QRroomaide)
+	) {
+		retval = retval | UA_ADMINALLOWED | UA_DELETEALLOWED | UA_POSTALLOWED | UA_REPLYALLOWED;
 	}
 
 NEWMSG:	/* By the way, we also check for the presence of new messages */
@@ -673,12 +732,9 @@ void cmd_lrms(char *argbuf)
 	if (!IsEmptyStr(argbuf))
 		FloorBeingSearched = extract_int(argbuf, 0);
 
-	if (CtdlAccessCheck(ac_logged_in)) return;
+	if (CtdlAccessCheck(ac_logged_in_or_guest)) return;
 
-	if (CtdlGetUser(&CC->user, CC->curr_user)) {
-		cprintf("%d Can't locate user!\n", ERROR + INTERNAL_ERROR);
-		return;
-	}
+	CtdlGetUser(&CC->user, CC->curr_user);
 	cprintf("%d Accessible rooms:\n", LISTING_FOLLOWS);
 
 	CtdlForEachRoom(cmd_lrms_backend, &FloorBeingSearched);
@@ -711,12 +767,9 @@ void cmd_lkra(char *argbuf)
 	if (!IsEmptyStr(argbuf))
 		FloorBeingSearched = extract_int(argbuf, 0);
 
-	if (CtdlAccessCheck(ac_logged_in)) return;
+	if (CtdlAccessCheck(ac_logged_in_or_guest)) return;
 	
-	if (CtdlGetUser(&CC->user, CC->curr_user)) {
-		cprintf("%d Can't locate user!\n", ERROR + INTERNAL_ERROR);
-		return;
-	}
+	CtdlGetUser(&CC->user, CC->curr_user);
 	cprintf("%d Known rooms:\n", LISTING_FOLLOWS);
 
 	CtdlForEachRoom(cmd_lkra_backend, &FloorBeingSearched);
@@ -747,7 +800,7 @@ void cmd_lprm(char *argbuf)
 	if (!IsEmptyStr(argbuf))
 		FloorBeingSearched = extract_int(argbuf, 0);
 
-	cprintf("%d Publiic rooms:\n", LISTING_FOLLOWS);
+	cprintf("%d Public rooms:\n", LISTING_FOLLOWS);
 
 	CtdlForEachRoom(cmd_lprm_backend, &FloorBeingSearched);
 	cprintf("000\n");
@@ -780,12 +833,9 @@ void cmd_lkrn(char *argbuf)
 	if (!IsEmptyStr(argbuf))
 		FloorBeingSearched = extract_int(argbuf, 0);
 
-	if (CtdlAccessCheck(ac_logged_in)) return;
+	if (CtdlAccessCheck(ac_logged_in_or_guest)) return;
 	
-	if (CtdlGetUser(&CC->user, CC->curr_user)) {
-		cprintf("%d Can't locate user!\n", ERROR + INTERNAL_ERROR);
-		return;
-	}
+	CtdlGetUser(&CC->user, CC->curr_user);
 	cprintf("%d Rooms w/ new msgs:\n", LISTING_FOLLOWS);
 
 	CtdlForEachRoom(cmd_lkrn_backend, &FloorBeingSearched);
@@ -819,12 +869,9 @@ void cmd_lkro(char *argbuf)
 	if (!IsEmptyStr(argbuf))
 		FloorBeingSearched = extract_int(argbuf, 0);
 
-	if (CtdlAccessCheck(ac_logged_in)) return;
+	if (CtdlAccessCheck(ac_logged_in_or_guest)) return;
 	
-	if (CtdlGetUser(&CC->user, CC->curr_user)) {
-		cprintf("%d Can't locate user!\n", ERROR + INTERNAL_ERROR);
-		return;
-	}
+	CtdlGetUser(&CC->user, CC->curr_user);
 	cprintf("%d Rooms w/o new msgs:\n", LISTING_FOLLOWS);
 
 	CtdlForEachRoom(cmd_lkro_backend, &FloorBeingSearched);
@@ -858,12 +905,9 @@ void cmd_lzrm(char *argbuf)
 	if (!IsEmptyStr(argbuf))
 		FloorBeingSearched = extract_int(argbuf, 0);
 
-	if (CtdlAccessCheck(ac_logged_in)) return;
+	if (CtdlAccessCheck(ac_logged_in_or_guest)) return;
 	
-	if (CtdlGetUser(&CC->user, CC->curr_user)) {
-		cprintf("%d Can't locate user!\n", ERROR + INTERNAL_ERROR);
-		return;
-	}
+	CtdlGetUser(&CC->user, CC->curr_user);
 	cprintf("%d Zapped rooms:\n", LISTING_FOLLOWS);
 
 	CtdlForEachRoom(cmd_lzrm_backend, &FloorBeingSearched);
@@ -1001,7 +1045,7 @@ void CtdlUserGoto(char *where, int display_result, int transiently,
 
 	if (retmsgs != NULL) *retmsgs = total_messages;
 	if (retnew != NULL) *retnew = new_messages;
-	CtdlLogPrintf(CTDL_DEBUG, "<%s> %d new of %d total messages\n",
+	syslog(LOG_DEBUG, "<%s> %d new of %d total messages\n",
 		CC->room.QRname,
 		new_messages, total_messages
 	);
@@ -1079,7 +1123,7 @@ void cmd_goto(char *gargs)
 	char password[32];
 	int transiently = 0;
 
-	if (CtdlAccessCheck(ac_logged_in)) return;
+	if (CtdlAccessCheck(ac_logged_in_or_guest)) return;
 
 	extract_token(towhere, gargs, 0, '|', sizeof towhere);
 	extract_token(password, gargs, 1, '|', sizeof password);
@@ -1144,7 +1188,7 @@ void cmd_goto(char *gargs)
 				   ((ra & UA_KNOWN) == 0) &&
 			           (CC->user.axlevel < AxAideU)
                                   ) {
-				CtdlLogPrintf(CTDL_DEBUG, "Failed to acquire private room\n");
+				syslog(LOG_DEBUG, "Failed to acquire private room\n");
 			} else {
 				memcpy(&CC->room, &QRscratch,
 					sizeof(struct ctdlroom));
@@ -1172,9 +1216,10 @@ void cmd_whok(char *cmdbuf)
 		cdb_free(cdbus);
 
 		CtdlRoomAccess(&CC->room, &temp, &ra, NULL);
-		if ((CC->room.QRflags & QR_INUSE)
-		    && (ra & UA_KNOWN)
-		    )
+		if ((!IsEmptyStr(temp.fullname)) && 
+		    (CC->room.QRflags & QR_INUSE) &&
+		    (ra & UA_KNOWN)
+			)
 			cprintf("%s\n", temp.fullname);
 	}
 	cprintf("000\n");
@@ -1323,7 +1368,7 @@ int CtdlRenameRoom(char *old_name, char *new_name, int new_floor) {
 	long owner = 0L;
 	char actual_old_name[ROOMNAMELEN];
 
-	CtdlLogPrintf(CTDL_DEBUG, "CtdlRenameRoom(%s, %s, %d)\n",
+	syslog(LOG_DEBUG, "CtdlRenameRoom(%s, %s, %d)\n",
 		old_name, new_name, new_floor);
 
 	if (new_floor >= 0) {
@@ -1414,11 +1459,11 @@ int CtdlRenameRoom(char *old_name, char *new_name, int new_floor) {
 		lgetfloor(&flbuf, old_floor);
 		--flbuf.f_ref_count;
 		lputfloor(&flbuf, old_floor);
-		CtdlLogPrintf(CTDL_DEBUG, "Reference count for floor %d is now %d\n", old_floor, flbuf.f_ref_count);
+		syslog(LOG_DEBUG, "Reference count for floor %d is now %d\n", old_floor, flbuf.f_ref_count);
 		lgetfloor(&flbuf, new_floor);
 		++flbuf.f_ref_count;
 		lputfloor(&flbuf, new_floor);
-		CtdlLogPrintf(CTDL_DEBUG, "Reference count for floor %d is now %d\n", new_floor, flbuf.f_ref_count);
+		syslog(LOG_DEBUG, "Reference count for floor %d is now %d\n", new_floor, flbuf.f_ref_count);
 	}
 
 	/* ...and everybody say "YATTA!" */	
@@ -1667,7 +1712,7 @@ void CtdlScheduleRoomForDeletion(struct ctdlroom *qrbuf)
 	char old_name[ROOMNAMELEN];
 	static int seq = 0;
 
-	CtdlLogPrintf(CTDL_NOTICE, "Scheduling room <%s> for deletion\n",
+	syslog(LOG_NOTICE, "Scheduling room <%s> for deletion\n",
 		qrbuf->QRname);
 
 	safestrncpy(old_name, qrbuf->QRname, sizeof old_name);
@@ -1705,7 +1750,7 @@ void CtdlDeleteRoom(struct ctdlroom *qrbuf)
 	char filename[100];
 	/* TODO: filename magic? does this realy work? */
 
-	CtdlLogPrintf(CTDL_NOTICE, "Deleting room <%s>\n", qrbuf->QRname);
+	syslog(LOG_NOTICE, "Deleting room <%s>\n", qrbuf->QRname);
 
 	/* Delete the info file */
 	assoc_file_name(filename, sizeof filename, qrbuf, ctdl_info_dir);
@@ -1836,11 +1881,11 @@ unsigned CtdlCreateRoom(char *new_room_name,
 	struct floor flbuf;
 	visit vbuf;
 
-	CtdlLogPrintf(CTDL_DEBUG, "CtdlCreateRoom(name=%s, type=%d, view=%d)\n",
+	syslog(LOG_DEBUG, "CtdlCreateRoom(name=%s, type=%d, view=%d)\n",
 		new_room_name, new_room_type, new_room_view);
 
 	if (CtdlGetRoom(&qrbuf, new_room_name) == 0) {
-		CtdlLogPrintf(CTDL_DEBUG, "%s already exists.\n", new_room_name);
+		syslog(LOG_DEBUG, "%s already exists.\n", new_room_name);
 		return(0);
 	}
 
@@ -1869,12 +1914,20 @@ unsigned CtdlCreateRoom(char *new_room_name,
 	}
 
 	/* If the room is private, and the system administrator has elected
-	 * to automatically grant room aide privileges, do so now; otherwise,
-	 * set the room aide to undefined.
+	 * to automatically grant room aide privileges, do so now.
 	 */
 	if ((qrbuf.QRflags & QR_PRIVATE) && (CREATAIDE == 1)) {
 		qrbuf.QRroomaide = CC->user.usernum;
-	} else {
+	}
+	/* Blog owners automatically become room aides of their blogs.
+	 * (In the future we will offer a site-wide configuration setting to suppress this behavior.)
+	 */
+	else if (new_room_view == VIEW_BLOG) {
+		qrbuf.QRroomaide = CC->user.usernum;
+	}
+	/* Otherwise, set the room aide to undefined.
+	 */
+	else {
 		qrbuf.QRroomaide = (-1L);
 	}
 
@@ -2048,9 +2101,9 @@ void cmd_einf(char *ok)
 		return;
 	}
 	assoc_file_name(infofilename, sizeof infofilename, &CC->room, ctdl_info_dir);
-	CtdlLogPrintf(CTDL_DEBUG, "opening\n");
+	syslog(LOG_DEBUG, "opening\n");
 	fp = fopen(infofilename, "w");
-	CtdlLogPrintf(CTDL_DEBUG, "checking\n");
+	syslog(LOG_DEBUG, "checking\n");
 	if (fp == NULL) {
 		cprintf("%d Cannot open %s: %s\n",
 		  ERROR + INTERNAL_ERROR, infofilename, strerror(errno));
@@ -2080,7 +2133,7 @@ void cmd_lflr(char *gargs)
 	int a;
 	struct floor flbuf;
 
-	if (CtdlAccessCheck(ac_logged_in)) return;
+	if (CtdlAccessCheck(ac_logged_in_or_guest)) return;
 
 	cprintf("%d Known floors:\n", LISTING_FOLLOWS);
 
@@ -2241,28 +2294,28 @@ void cmd_eflr(char *argbuf)
 CTDL_MODULE_INIT(room_ops)
 {
 	if (!threading) {
-		CtdlRegisterProtoHook(cmd_lrms, "LRMS", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_lkra, "LKRA", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_lkrn, "LKRN", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_lkro, "LKRO", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_lzrm, "LZRM", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_lprm, "LPRM", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_goto, "GOTO", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_whok, "WHOK", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_rdir, "RDIR", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_getr, "GETR", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_setr, "SETR", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_geta, "GETA", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_seta, "SETA", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_rinf, "RINF", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_kill, "KILL", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_cre8, "CRE8", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_einf, "EINF", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_lflr, "LFLR", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_cflr, "CFLR", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_kflr, "KFLR", "Autoconverted. TODO: document me.");
-		CtdlRegisterProtoHook(cmd_eflr, "EFLR", "Autoconverted. TODO: document me.");
+		CtdlRegisterProtoHook(cmd_lrms, "LRMS", "List rooms");
+		CtdlRegisterProtoHook(cmd_lkra, "LKRA", "List all known rooms");
+		CtdlRegisterProtoHook(cmd_lkrn, "LKRN", "List known rooms with new messages");
+		CtdlRegisterProtoHook(cmd_lkro, "LKRO", "List known rooms without new messages");
+		CtdlRegisterProtoHook(cmd_lzrm, "LZRM", "List zapped rooms");
+		CtdlRegisterProtoHook(cmd_lprm, "LPRM", "List public rooms");
+		CtdlRegisterProtoHook(cmd_goto, "GOTO", "Goto a named room");
+		CtdlRegisterProtoHook(cmd_whok, "WHOK", "List users who know this room");
+		CtdlRegisterProtoHook(cmd_rdir, "RDIR", "List files in room directory");
+		CtdlRegisterProtoHook(cmd_getr, "GETR", "Get room parameters");
+		CtdlRegisterProtoHook(cmd_setr, "SETR", "Set room parameters");
+		CtdlRegisterProtoHook(cmd_geta, "GETA", "Get the room aide name");
+		CtdlRegisterProtoHook(cmd_seta, "SETA", "Set the room aide for this room");
+		CtdlRegisterProtoHook(cmd_rinf, "RINF", "Fetch room info file");
+		CtdlRegisterProtoHook(cmd_kill, "KILL", "Kill (delete) the current room");
+		CtdlRegisterProtoHook(cmd_cre8, "CRE8", "Create a new room");
+		CtdlRegisterProtoHook(cmd_einf, "EINF", "Enter info file for the current room");
+		CtdlRegisterProtoHook(cmd_lflr, "LFLR", "List all known floors");
+		CtdlRegisterProtoHook(cmd_cflr, "CFLR", "Create a new floor");
+		CtdlRegisterProtoHook(cmd_kflr, "KFLR", "Kill a floor");
+		CtdlRegisterProtoHook(cmd_eflr, "EFLR", "Edit a floor");
 	}
         /* return our Subversion id for the Log */
-	return "$Id$";
+	return "room_ops";
 }
