@@ -1,8 +1,5 @@
 /*
- * $Id$
- *
  * Citadel "system dependent" stuff.
- * See COPYING for copyright information.
  *
  * Here's where we (hopefully) have most parts of the Citadel server that
  * would need to be altered to run the server in a non-POSIX environment.
@@ -10,6 +7,21 @@
  * If we ever port to a different platform and either have multiple
  * variants of this file or simply load it up with #ifdefs.
  *
+ * Copyright (c) 1987-2011 by the citadel.org team
+ *
+ * This program is open source software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 
 #include "sysdep.h"
@@ -77,126 +89,6 @@
 #include "control.h"
 
 
-#ifdef DEBUG_MEMORY_LEAKS
-struct igheap {
-	struct igheap *next;
-	char file[32];
-	int line;
-	void *block;
-};
-
-struct igheap *igheap = NULL;
-#endif
-
-
-int verbosity = DEFAULT_VERBOSITY;		/* Logging level */
-
-int syslog_facility = LOG_DAEMON;
-int enable_syslog = 0;
-int print_to_logfile = 1;
-
-/*
- * CtdlLogPrintf()  ...   Write logging information
- */
-void CtdlLogPrintf(enum LogLevel loglevel, const char *format, ...) {   
-	va_list arg_ptr;
-	va_start(arg_ptr, format);
-	vCtdlLogPrintf(loglevel, format, arg_ptr);
-	va_end(arg_ptr);
-}
-
-void vCtdlLogPrintf(enum LogLevel loglevel, const char *format, va_list arg_ptr)
-{
-
-	if (enable_syslog) {
-		vsyslog((syslog_facility | loglevel), format, arg_ptr);
-	}
-
-	/* stderr output code */
-	if (enable_syslog || !print_to_logfile) return;
-
-	/* if we run in forground and syslog is disabled, log to terminal */
-	if (loglevel <= verbosity) { 
-		struct timeval tv;
-		struct tm tim;
-		time_t unixtime;
-		StrBuf *lBuf;
-		CitContext *CCC = CC;
-		ThreadTSD *cTSD = CTP;
-		CtdlThreadNode *node = NULL;
-		long lwpid = 0;
-
-		if (cTSD != NULL) 
-			node = cTSD->self;
-		if ((node != NULL) && (node->reltid != 0))
-		{
-			lwpid = node->reltid;
-		}
-
-		gettimeofday(&tv, NULL);
-		/* Promote to time_t; types differ on some OSes (like darwin) */
-		unixtime = tv.tv_sec;
-		localtime_r(&unixtime, &tim);
-
-		if ((CCC != NULL) && (CCC != &masterCC))
-			lBuf = CCC->lBuf;
-		else 
-			lBuf = NewStrBuf();
-		if (lBuf == NULL) {
-			char buf[SIZ], buf2[SIZ];
-			
-			if ((CCC != NULL) && (CCC->cs_pid != 0)) {
-				sprintf(buf,
-					"%04d/%02d/%02d %2d:%02d:%02d.%06ld xx [%3d] ",
-					tim.tm_year + 1900, tim.tm_mon + 1,
-					tim.tm_mday, tim.tm_hour, tim.tm_min,
-					tim.tm_sec, (long)tv.tv_usec,
-					CCC->cs_pid);
-			} else {
-				sprintf(buf,
-					"%04d/%02d/%02d %2d:%02d:%02d.%06ld xx ",
-					tim.tm_year + 1900, tim.tm_mon + 1,
-					tim.tm_mday, tim.tm_hour, tim.tm_min,
-					tim.tm_sec, (long)tv.tv_usec);
-			}
-			vsnprintf(buf2, SIZ, format, arg_ptr);   
-
-			fprintf(stderr, ":%s%s", buf, buf2);
-		}
-		else {
-			StrBufPrintf(lBuf,
-				     "%04d/%02d/%02d %2d:%02d:%02d.%06ld ",
-				     tim.tm_year + 1900, tim.tm_mon + 1,
-				     tim.tm_mday, tim.tm_hour, tim.tm_min,
-				     tim.tm_sec, (long)tv.tv_usec);
-
-			
-			if (lwpid != 0)
-				StrBufAppendPrintf(lBuf,
-						   "[LWP:%d] ",
-						   lwpid);
-				
-			if (CCC != NULL) {
-				if (CCC->cs_pid != 0)
-					StrBufAppendPrintf(lBuf,
-							   "[%3d] ",
-							   CCC->cs_pid);
-				else if (CCC->user.usernum != 0)
-					StrBufAppendPrintf(lBuf,
-							   "[:%d] ",
-							   CCC->user.usernum);
-			}
-			StrBufVAppendPrintf(lBuf, format, arg_ptr);   
-			fwrite(ChrPtr(lBuf), 1, StrLength(lBuf), stderr);
-		}
-		fflush(stderr);
-		if (CCC == NULL) FreeStrBuf(&lBuf);
-
-	}
-}   
-
-
-
 /*
  * Signal handler to shut down the server.
  */
@@ -207,14 +99,9 @@ volatile int restart_server = 0;
 volatile int running_as_daemon = 0;
 
 static RETSIGTYPE signal_cleanup(int signum) {
-
-	if (CT)
-		CT->signal = signum;
-	else
-	{
-		CtdlLogPrintf(CTDL_DEBUG, "Caught signal %d; shutting down.\n", signum);
-		exit_signal = signum;
-	}
+	syslog(LOG_DEBUG, "Caught signal %d; shutting down.", signum);
+	exit_signal = signum;
+	server_shutting_down = 1;
 }
 
 static RETSIGTYPE signal_exit(int signum) {
@@ -251,33 +138,24 @@ void init_sysdep(void) {
 	 * CitContext structure (in the ContextList linked list) of the
 	 * session to which the calling thread is currently bound.
 	 */
-	if (citthread_key_create(&MyConKey, NULL) != 0) {
-		CtdlLogPrintf(CTDL_CRIT, "Can't create TSD key: %s\n",
-			strerror(errno));
+	if (pthread_key_create(&MyConKey, NULL) != 0) {
+		syslog(LOG_CRIT, "Can't create TSD key: %s", strerror(errno));
 	}
 
 	/*
-	 * The action for unexpected signals and exceptions should be to
-	 * call signal_cleanup() to gracefully shut down the server.
+	 * Interript, hangup, and terminate signals should cause the server
+	 * to gracefully clean up and shut down.
 	 */
 	sigemptyset(&set);
-	sigaddset(&set, SIGINT);		// intr = shutdown
-	// sigaddset(&set, SIGQUIT);		// quit = force quit
+	sigaddset(&set, SIGINT);
 	sigaddset(&set, SIGHUP);
 	sigaddset(&set, SIGTERM);
-	// sigaddset(&set, SIGSEGV);		// we want core dumps
-	// sigaddset(&set, SIGILL);		// we want core dumps
-	// sigaddset(&set, SIGBUS);
 	sigprocmask(SIG_UNBLOCK, &set, NULL);
 
-	signal(SIGINT, signal_cleanup);		// intr = shutdown
-	// signal(SIGQUIT, signal_cleanup);	// quit = force quit
+	signal(SIGINT, signal_cleanup);
 	signal(SIGHUP, signal_cleanup);
 	signal(SIGTERM, signal_cleanup);
 	signal(SIGUSR2, signal_exit);
-	// signal(SIGSEGV, signal_cleanup);	// we want coredumps
-	// signal(SIGILL, signal_cleanup);	// we want core dumps
-	// signal(SIGBUS, signal_cleanup);
 
 	/*
 	 * Do not shut down the server on broken pipe signals, otherwise the
@@ -288,90 +166,119 @@ void init_sysdep(void) {
 }
 
 
-
-
-/*
+/* 
  * This is a generic function to set up a master socket for listening on
- * a TCP port.  The server shuts down if the bind fails.
+ * a TCP port.  The server shuts down if the bind fails.  (IPv4/IPv6 version)
  *
+ * ip_addr 	IP address to bind
+ * port_number	port number to bind
+ * queue_len	number of incoming connections to allow in the queue
  */
-int ig_tcp_server(char *ip_addr, int port_number, int queue_len, char **errormessage)
+int ctdl_tcp_server(char *ip_addr, int port_number, int queue_len, char *errormessage)
 {
-	struct sockaddr_in sin;
-	int s, i;
-	int actual_queue_len;
+	struct protoent *p;
+	struct sockaddr_in6 sin6;
+	struct sockaddr_in sin4;
+	int s, i, b;
+	int ip_version = 6;
 
-	actual_queue_len = queue_len;
-	if (actual_queue_len < 5) actual_queue_len = 5;
+	memset(&sin6, 0, sizeof(sin6));
+	memset(&sin4, 0, sizeof(sin4));
+	sin6.sin6_family = AF_INET6;
+	sin4.sin_family = AF_INET;
 
-	memset(&sin, 0, sizeof(sin));
-	sin.sin_family = AF_INET;
-	sin.sin_port = htons((u_short)port_number);
-	if (ip_addr == NULL) {
-		sin.sin_addr.s_addr = INADDR_ANY;
+	if (	(ip_addr == NULL)							/* any IPv6 */
+		|| (IsEmptyStr(ip_addr))
+		|| (!strcmp(ip_addr, "*"))
+	) {
+		ip_version = 6;
+		sin6.sin6_addr = in6addr_any;
 	}
-	else {
-		sin.sin_addr.s_addr = inet_addr(ip_addr);
+	else if (!strcmp(ip_addr, "0.0.0.0"))						/* any IPv4 */
+	{
+		ip_version = 4;
+		sin4.sin_addr.s_addr = INADDR_ANY;
 	}
-										
-	if (sin.sin_addr.s_addr == !INADDR_ANY) {
-		sin.sin_addr.s_addr = INADDR_ANY;
+	else if ((strchr(ip_addr, '.')) && (!strchr(ip_addr, ':')))			/* specific IPv4 */
+	{
+		ip_version = 4;
+		if (inet_pton(AF_INET, ip_addr, &sin4.sin_addr) <= 0) {
+			snprintf(errormessage, SIZ,
+				 "Error binding to [%s] : %s", ip_addr, strerror(errno)
+			);
+			syslog(LOG_ALERT, "%s", errormessage);
+			return (-1);
+		}
+	}
+	else										/* specific IPv6 */
+	{
+		ip_version = 6;
+		if (inet_pton(AF_INET6, ip_addr, &sin6.sin6_addr) <= 0) {
+			snprintf(errormessage, SIZ,
+				 "Error binding to [%s] : %s", ip_addr, strerror(errno)
+			);
+			syslog(LOG_ALERT, "%s", errormessage);
+			return (-1);
+		}
 	}
 
-	s = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP);
+	if (port_number == 0) {
+		snprintf(errormessage, SIZ, "Can't start: no port number specified.");
+		syslog(LOG_ALERT, "%s", errormessage);
+		return (-1);
+	}
+	sin6.sin6_port = htons((u_short) port_number);
+	sin4.sin_port = htons((u_short) port_number);
 
+	p = getprotobyname("tcp");
+
+	s = socket( ((ip_version == 6) ? PF_INET6 : PF_INET), SOCK_STREAM, (p->p_proto));
 	if (s < 0) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, 
-				 "citserver: Can't create a socket: %s",
-				 strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
-		return(-1);
+		snprintf(errormessage, SIZ,
+			 "Can't create a listening socket: %s", strerror(errno)
+		);
+		syslog(LOG_ALERT, "%s", errormessage);
+		return (-1);
 	}
-
+	/* Set some socket options that make sense. */
 	i = 1;
 	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
 
-	if (bind(s, (struct sockaddr *)&sin, sizeof(sin)) < 0) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, 
-				 "citserver: Can't bind: %s",
-				 strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
-		close(s);
-		return(-1);
+	if (ip_version == 6) {
+		b = bind(s, (struct sockaddr *) &sin6, sizeof(sin6));
+	}
+	else {
+		b = bind(s, (struct sockaddr *) &sin4, sizeof(sin4));
 	}
 
-	/* set to nonblock - we need this for some obscure situations */
-	if (fcntl(s, F_SETFL, O_NONBLOCK) < 0) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, 
-				 "citserver: Can't set socket to non-blocking: %s",
-				 strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
-		close(s);
-		return(-1);
+	if (b < 0) {
+		snprintf(errormessage, SIZ,
+			 "Can't bind: %s", strerror(errno)
+		);
+		syslog(LOG_ALERT, "%s", errormessage);
+		return (-1);
 	}
 
-	if (listen(s, actual_queue_len) < 0) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, 
-				 "citserver: Can't listen: %s",
-				 strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
-		close(s);
-		return(-1);
-	}
+	fcntl(s, F_SETFL, O_NONBLOCK);
 
-	return(s);
+	if (listen(s, ((queue_len >= 5) ? queue_len : 5) ) < 0) {
+		snprintf(errormessage, SIZ,
+			 "Can't listen: %s", strerror(errno)
+		);
+		syslog(LOG_ALERT, "%s", errormessage);
+		return (-1);
+	}
+	return (s);
 }
 
 
 
+
+
 /*
  * Create a Unix domain socket and listen on it
  */
-int ig_uds_server(char *sockpath, int queue_len, char **errormessage)
+int ctdl_uds_server(char *sockpath, int queue_len, char *errormessage)
 {
 	struct sockaddr_un addr;
 	int s;
@@ -386,10 +293,10 @@ int ig_uds_server(char *sockpath, int queue_len, char **errormessage)
 
 	i = unlink(sockpath);
 	if ((i != 0) && (errno != ENOENT)) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, "citserver: can't unlink %s: %s",
-			sockpath, strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
+		snprintf(errormessage, SIZ, "citserver: can't unlink %s: %s",
+			sockpath, strerror(errno)
+		);
+		syslog(LOG_EMERG, "%s", errormessage);
 		return(-1);
 	}
 
@@ -399,40 +306,36 @@ int ig_uds_server(char *sockpath, int queue_len, char **errormessage)
 
 	s = socket(AF_UNIX, SOCK_STREAM, 0);
 	if (s < 0) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, 
+		snprintf(errormessage, SIZ, 
 			 "citserver: Can't create a socket: %s",
 			 strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
+		syslog(LOG_EMERG, "%s", errormessage);
 		return(-1);
 	}
 
 	if (bind(s, (struct sockaddr *)&addr, sizeof(addr)) < 0) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, 
+		snprintf(errormessage, SIZ, 
 			 "citserver: Can't bind: %s",
 			 strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
+		syslog(LOG_EMERG, "%s", errormessage);
 		return(-1);
 	}
 
 	/* set to nonblock - we need this for some obscure situations */
 	if (fcntl(s, F_SETFL, O_NONBLOCK) < 0) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, 
+		snprintf(errormessage, SIZ, 
 			 "citserver: Can't set socket to non-blocking: %s",
 			 strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
+		syslog(LOG_EMERG, "%s", errormessage);
 		close(s);
 		return(-1);
 	}
 
 	if (listen(s, actual_queue_len) < 0) {
-		*errormessage = (char*) malloc(SIZ + 1);
-		snprintf(*errormessage, SIZ, 
+		snprintf(errormessage, SIZ, 
 			 "citserver: Can't listen: %s",
 			 strerror(errno));
-		CtdlLogPrintf(CTDL_EMERG, "%s\n", *errormessage);
+		syslog(LOG_EMERG, "%s", errormessage);
 		return(-1);
 	}
 
@@ -500,6 +403,23 @@ static void flush_client_inbuf(void)
 }
 */
 
+
+/*
+ * client_close()	...	close the client socket
+ */
+void client_close(void) {
+	CitContext *CCC = CC;
+
+	if (!CCC) return;
+	if (CCC->client_socket < 0) return;
+	syslog(LOG_DEBUG, "Closing socket %d", CCC->client_socket);
+	close(CCC->client_socket);
+	CCC->client_socket = -1 ;
+}
+
+
+
+
 /*
  * client_write()   ...    Send binary data to the client.
  */
@@ -560,36 +480,40 @@ int client_write(const char *buf, int nbytes)
 			if (select(1, NULL, &wset, NULL, NULL) == -1) {
 				if (errno == EINTR)
 				{
-					CtdlLogPrintf(CTDL_DEBUG, "client_write(%d bytes) select() interrupted.\n", nbytes-bytes_written);
-					if (CtdlThreadCheckStop()) {
-						CC->kill_me = 1;
+					syslog(LOG_DEBUG, "client_write(%d bytes) select() interrupted.",
+						nbytes-bytes_written
+					);
+					if (server_shutting_down) {
+						CC->kill_me = KILLME_SELECT_INTERRUPTED;
 						return (-1);
 					} else {
 						/* can't trust fd's and stuff so we need to re-create them */
 						continue;
 					}
 				} else {
-					CtdlLogPrintf(CTDL_ERR,
-						"client_write(%d bytes) select failed: %s (%d)\n",
+					syslog(LOG_ERR,
+						"client_write(%d bytes) select failed: %s (%d)",
 						nbytes - bytes_written,
-						strerror(errno), errno);
+						strerror(errno), errno
+					);
 					cit_backtrace();
-					Ctx->kill_me = 1;
+					client_close();
+					Ctx->kill_me = KILLME_SELECT_FAILED;
 					return -1;
 				}
 			}
 		}
 
-		retval = write(Ctx->client_socket, &buf[bytes_written],
-			nbytes - bytes_written);
+		retval = write(Ctx->client_socket, &buf[bytes_written], nbytes - bytes_written);
 		if (retval < 1) {
-			CtdlLogPrintf(CTDL_ERR,
-				"client_write(%d bytes) failed: %s (%d)\n",
+			syslog(LOG_ERR,
+				"client_write(%d bytes) failed: %s (%d)",
 				nbytes - bytes_written,
-				strerror(errno), errno);
+				strerror(errno), errno
+			);
 			cit_backtrace();
-			// CtdlLogPrintf(CTDL_DEBUG, "Tried to send: %s",  &buf[bytes_written]);
-			Ctx->kill_me = 1;
+			client_close();
+			Ctx->kill_me = KILLME_WRITE_FAILED;
 			return -1;
 		}
 		bytes_written = bytes_written + retval;
@@ -639,16 +563,58 @@ int client_read_blob(StrBuf *Target, int bytes, int timeout)
 
 #ifdef HAVE_OPENSSL
 	if (CCC->redirect_ssl) {
+#ifdef BIGBAD_IODBG
+		int rv = 0;
+		char fn [SIZ];
+		FILE *fd;
+		
+		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
+			
+		fd = fopen(fn, "a+");
+		fprintf(fd, "Reading BLOB: BufSize: %d ",
+			bytes);
+		rv = fwrite(ChrPtr(Target), StrLength(Target), 1, fd);
+		fprintf(fd, "]\n");
+		
+			
+		fclose(fd);
+#endif
 		retval = client_read_sslblob(Target, bytes, timeout);
 		if (retval < 0) {
-			CtdlLogPrintf(CTDL_CRIT, 
-				      "%s failed\n",
-				      __FUNCTION__);
+			syslog(LOG_CRIT, "client_read_blob() failed");
 		}
+#ifdef BIGBAD_IODBG
+		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
+		
+		fd = fopen(fn, "a+");
+		fprintf(fd, "Read: %d BufContent: [",
+			StrLength(Target));
+		rv = fwrite(ChrPtr(Target), StrLength(Target), 1, fd);
+		fprintf(fd, "]\n");
+		
+		
+		fclose(fd);
+#endif
 	}
 	else 
 #endif
 	{
+#ifdef BIGBAD_IODBG
+		int rv = 0;
+		char fn [SIZ];
+		FILE *fd;
+		
+		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
+			
+		fd = fopen(fn, "a+");
+		fprintf(fd, "Reading BLOB: BufSize: %d ",
+			bytes);
+		rv = fwrite(ChrPtr(Target), StrLength(Target), 1, fd);
+		fprintf(fd, "]\n");
+		
+			
+		fclose(fd);
+#endif
 		retval = StrBufReadBLOBBuffered(Target, 
 						CCC->ReadBuf,
 						&CCC->Pos,
@@ -658,32 +624,20 @@ int client_read_blob(StrBuf *Target, int bytes, int timeout)
 						O_TERM,
 						&Error);
 		if (retval < 0) {
-			CtdlLogPrintf(CTDL_CRIT, 
-				      "%s failed: %s\n",
-				      __FUNCTION__, 
-				      Error);
+			syslog(LOG_CRIT, "client_read_blob() failed: %s", Error);
+			client_close();
 			return retval;
 		}
-		else
-		{
 #ifdef BIGBAD_IODBG
-			int rv = 0;
-			char fn [SIZ];
-			FILE *fd;
-			
-			snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
-			
-			fd = fopen(fn, "a+");
-			fprintf(fd, "Read: BufSize: %d BufContent: [",
-				StrLength(Target));
-			rv = fwrite(ChrPtr(Target), StrLength(Target), 1, fd);
-			fprintf(fd, "]\n");
-			
-			
-			fclose(fd);
+		snprintf(fn, SIZ, "/tmp/foolog_%s.%d", CCC->ServiceName, CCC->cs_pid);
+		
+		fd = fopen(fn, "a+");
+		fprintf(fd, "Read: %d BufContent: [",
+			StrLength(Target));
+		rv = fwrite(ChrPtr(Target), StrLength(Target), 1, fd);
+		fprintf(fd, "]\n");
+		fclose(fd);
 #endif
-
-		}
 	}
 	return retval;
 }
@@ -773,7 +727,7 @@ int client_read_to(char *buf, int bytes, int timeout)
 
 int HaveMoreLinesWaiting(CitContext *CCC)
 {
-	if ((CCC->kill_me == 1) || (
+	if ((CCC->kill_me != 0) || (
 	    (CCC->Pos == NULL) && 
 	    (StrLength(CCC->ReadBuf) == 0) && 
 	    (CCC->client_socket != -1)) )
@@ -849,10 +803,9 @@ int CtdlClientGetLine(StrBuf *Target)
                         StrLength(Target), ChrPtr(Target));
                 fclose(fd);
 
-		if (rc < 0)
-			CtdlLogPrintf(CTDL_CRIT, 
-				      "%s failed\n",
-				      __FUNCTION__);
+		if (rc < 0) {
+			syslog(LOG_CRIT, "CtdlClientGetLine() failed");
+		}
 #endif
 		return rc;
 	}
@@ -907,11 +860,9 @@ int CtdlClientGetLine(StrBuf *Target)
                         StrLength(Target), ChrPtr(Target));
                 fclose(fd);
 
-		if ((rc < 0) && (Error != NULL))
-			CtdlLogPrintf(CTDL_CRIT, 
-				      "%s failed: %s\n",
-				      __FUNCTION__,
-				      Error);
+		if ((rc < 0) && (Error != NULL)) {
+			syslog(LOG_CRIT, "CtdlClientGetLine() failed: %s", Error);
+		}
 #endif
 		return rc;
 	}
@@ -973,13 +924,13 @@ void close_masters (void)
 
 		if (serviceptr->tcp_port > 0)
 		{
-			CtdlLogPrintf(CTDL_INFO, "Closing listener on port %d\n",
+			syslog(LOG_INFO, "Closing listener on port %d\n",
 				serviceptr->tcp_port);
 			serviceptr->tcp_port = 0;
 		}
 		
 		if (serviceptr->sockpath != NULL)
-			CtdlLogPrintf(CTDL_INFO, "Closing listener on '%s'\n",
+			syslog(LOG_INFO, "Closing listener on '%s'\n",
 				serviceptr->sockpath);
 
 		close(serviceptr->msock);
@@ -1015,6 +966,7 @@ void sysdep_master_cleanup(void) {
 	CtdlDestroySessionHooks();
 	CtdlDestroyServiceHook();
 	CtdlDestroyRoomHooks();
+	CtdlDestroySearchHooks();
 	#ifdef HAVE_BACKTRACE
 ///	eCrash_Uninit();
 	#endif
@@ -1049,7 +1001,7 @@ void start_daemon(int unused) {
 	 * to be reused for other files.
 	 */
 	if (chdir(ctdl_run_dir) != 0)
-		CtdlLogPrintf(CTDL_EMERG, 
+		syslog(LOG_EMERG, 
 			      "unable to change into directory [%s]: %s", 
 			      ctdl_run_dir, strerror(errno));
 
@@ -1067,7 +1019,7 @@ void start_daemon(int unused) {
         if ((freopen("/dev/null", "r", stdin) != stdin) || 
 	    (freopen("/dev/null", "w", stdout) != stdout) || 
 	    (freopen("/dev/null", "w", stderr) != stderr))
-		CtdlLogPrintf(CTDL_EMERG, 
+		syslog(LOG_EMERG, 
 			      "unable to reopen stdin/out/err %s", 
 			      strerror(errno));
 		
@@ -1129,7 +1081,7 @@ void checkcrash(void)
 		StrBuf *CrashMail;
 
 		CrashMail = NewStrBuf();
-		CtdlLogPrintf(CTDL_ALERT, "Posting crash message\n");
+		syslog(LOG_ALERT, "Posting crash message\n");
 		StrBufPrintf(CrashMail, 
 			" \n"
 			" The Citadel server process (citserver) terminated unexpectedly."
@@ -1176,32 +1128,7 @@ int convert_login(char NameToConvert[]) {
 /* 
  * This loop just keeps going and going and going...
  */
-/*
- * FIXME:
- * This current implimentation of worker_thread creates a bottle neck in several situations
- * The first thing to remember is that a single thread can handle more than one connection at a time.
- * More threads mean less memory for the system to run in.
- * So for efficiency we want every thread to be doing something useful or waiting in the main loop for
- * something to happen anywhere.
- * This current implimentation requires worker threads to wait in other locations, after it has
- * been committed to a single connection which is very wasteful.
- * As an extreme case consider this:
- * A slow client connects and this slow client sends only one character each second.
- * With this current implimentation a single worker thread is dispatched to handle that connection
- * until such times as the client timeout expires, an error occurs on the socket or the client
- * completes its transmission.
- * THIS IS VERY BAD since that thread could have handled a read from many more clients in each one
- * second interval between chars.
- *
- * It is my intention to re-write this code and the associated client_getln, client_read functions
- * to allow any thread to read data on behalf of any connection (context).
- * To do this I intend to have this main loop read chars into a buffer stored in the context.
- * Once the correct criteria for a full buffer is met then we will dispatch a thread to 
- * process it.
- * This worker thread loop also needs to be able to handle binary data.
- */
- 
-void *worker_thread(void *arg) {
+void *worker_thread(void *blah) {
 	int highest;
 	CitContext *ptr;
 	CitContext *bind_me = NULL;
@@ -1209,9 +1136,14 @@ void *worker_thread(void *arg) {
 	int retval = 0;
 	struct timeval tv;
 	int force_purge = 0;
-	
+	struct ServiceFunctionHook *serviceptr;
+	int ssock;                      /* Descriptor for client socket */
+	CitContext *con = NULL;         /* Temporary context pointer */
+	int i;
 
-	while (!CtdlThreadCheckStop()) {
+	++num_workers;
+
+	while (!server_shutting_down) {
 
 		/* make doubly sure we're not holding any stale db handles
 		 * which might cause a deadlock.
@@ -1224,18 +1156,25 @@ do_select:	force_purge = 0;
 		FD_ZERO(&readfds);
 		highest = 0;
 
+		/* First, add the various master sockets to the fdset. */
+		for (serviceptr = ServiceHookTable; serviceptr != NULL; serviceptr = serviceptr->next ) {
+			FD_SET(serviceptr->msock, &readfds);
+			if (serviceptr->msock > highest) {
+				highest = serviceptr->msock;
+			}
+		}
+
+		/* Next, add all of the client sockets. */
 		begin_critical_section(S_SESSION_TABLE);
 		for (ptr = ContextList; ptr != NULL; ptr = ptr->next) {
-			int client_socket;
-			client_socket = ptr->client_socket;
-			/* Dont select on dead sessions only truly idle ones */
-			if ((ptr->state == CON_IDLE) && 
-			    (CC->kill_me == 0) &&
-			    (client_socket != -1))
-			{
-				FD_SET(client_socket, &readfds);
-				if (client_socket > highest)
-					highest = client_socket;
+			/* Don't select on dead sessions, only truly idle ones */
+			if (	(ptr->state == CON_IDLE)
+				&& (ptr->kill_me == 0)
+				&& (ptr->client_socket != -1)
+			) {
+				FD_SET(ptr->client_socket, &readfds);
+				if (ptr->client_socket > highest)
+					highest = ptr->client_socket;
 			}
 			if ((bind_me == NULL) && (ptr->state == CON_READY)) {
 				bind_me = ptr;
@@ -1259,35 +1198,89 @@ do_select:	force_purge = 0;
 		 * ahead and get ready to select().
 		 */
 
-		if (!CtdlThreadCheckStop()) {
+		if (!server_shutting_down) {
 			tv.tv_sec = 1;		/* wake up every second if no input */
 			tv.tv_usec = 0;
-			retval = CtdlThreadSelect(highest + 1, &readfds, NULL, NULL, &tv);
+			retval = select(highest + 1, &readfds, NULL, NULL, &tv);
 		}
-		else
+		else {
+			--num_workers;
 			return NULL;
+		}
 
 		/* Now figure out who made this select() unblock.
 		 * First, check for an error or exit condition.
 		 */
 		if (retval < 0) {
 			if (errno == EBADF) {
-				CtdlLogPrintf(CTDL_NOTICE, "select() failed: (%s)\n",
-					strerror(errno));
+				syslog(LOG_NOTICE, "select() failed: (%s)\n", strerror(errno));
 				goto do_select;
 			}
 			if (errno != EINTR) {
-				CtdlLogPrintf(CTDL_EMERG, "Exiting (%s)\n", strerror(errno));
-				CtdlThreadStopAll();
+				syslog(LOG_EMERG, "Exiting (%s)\n", strerror(errno));
+				server_shutting_down = 1;
 				continue;
 			} else {
-				CtdlLogPrintf(CTDL_DEBUG, "Interrupted CtdlThreadSelect.\n");
-				if (CtdlThreadCheckStop()) return(NULL);
+				syslog(LOG_DEBUG, "Interrupted select()\n");
+				if (server_shutting_down) {
+					--num_workers;
+					return(NULL);
+				}
 				goto do_select;
 			}
 		}
-		else if(retval == 0) {
-			if (CtdlThreadCheckStop()) return(NULL);
+		else if (retval == 0) {
+			if (server_shutting_down) {
+				--num_workers;
+				return(NULL);
+			}
+		}
+
+		/* Next, check to see if it's a new client connecting * on a master socket. */
+
+		else if ((retval > 0) && (!server_shutting_down)) for (serviceptr = ServiceHookTable; serviceptr != NULL; serviceptr = serviceptr->next) {
+
+			if (FD_ISSET(serviceptr->msock, &readfds)) {
+				ssock = accept(serviceptr->msock, NULL, 0);
+				if (ssock >= 0) {
+					syslog(LOG_DEBUG, "New client socket %d", ssock);
+
+					/* The master socket is non-blocking but the client
+					 * sockets need to be blocking, otherwise certain
+					 * operations barf on FreeBSD.  Not a fatal error.
+					 */
+					if (fcntl(ssock, F_SETFL, 0) < 0) {
+						syslog(LOG_EMERG,
+							"citserver: Can't set socket to blocking: %s\n",
+							strerror(errno));
+					}
+
+					/* New context will be created already
+					 * set up in the CON_EXECUTING state.
+					 */
+					con = CreateNewContext();
+
+					/* Assign our new socket number to it. */
+					con->client_socket = ssock;
+					con->h_command_function = serviceptr->h_command_function;
+					con->h_async_function = serviceptr->h_async_function;
+					con->h_greeting_function = serviceptr->h_greeting_function;
+					con->ServiceName = serviceptr->ServiceName;
+					
+					/* Determine whether it's a local socket */
+					if (serviceptr->sockpath != NULL) {
+						con->is_local_socket = 1;
+					}
+	
+					/* Set the SO_REUSEADDR socket option */
+					i = 1;
+					setsockopt(ssock, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i));
+					con->state = CON_GREETING;
+					retval--;
+					if (retval == 0)
+						break;
+				}
+			}
 		}
 
 		/* It must be a client socket.  Find a context that has data
@@ -1324,6 +1317,7 @@ do_select:	force_purge = 0;
 
 SKIP_SELECT:
 		/* We're bound to a session */
+		++active_workers;
 		if (bind_me != NULL) {
 			become_session(bind_me);
 
@@ -1357,141 +1351,20 @@ SKIP_SELECT:
 
 		dead_session_purge(force_purge);
 		do_housekeeping();
+		--active_workers;
 	}
-	/* If control reaches this point, the server is shutting down */	
+
+	/* If control reaches this point, the server is shutting down */
+	--num_workers;
 	return(NULL);
 }
 
 
 
 
-/*
- * A function to handle selecting on master sockets.
- * In other words it handles new connections.
- * It is a thread.
- */
-void *select_on_master (void *arg)
-{
-	struct ServiceFunctionHook *serviceptr;
-	fd_set master_fds;
-	int highest;
-	struct timeval tv;
-	int ssock;			/* Descriptor for client socket */
-	CitContext *con= NULL;	/* Temporary context pointer */
-	int m;
-	int i;
-	int retval;
-	struct CitContext select_on_master_CC;
 
-	CtdlFillSystemContext(&select_on_master_CC, "select_on_master");
-	citthread_setspecific(MyConKey, (void *)&select_on_master_CC);
 
-	while (!CtdlThreadCheckStop()) {
-		/* Initialize the fdset. */
-		FD_ZERO(&master_fds);
-		highest = 0;
 
-		/* First, add the various master sockets to the fdset. */
-		for (serviceptr = ServiceHookTable; serviceptr != NULL;
-	    	serviceptr = serviceptr->next ) {
-			m = serviceptr->msock;
-			FD_SET(m, &master_fds);
-			if (m > highest) {
-				highest = m;
-			}
-		}
-
-		if (!CtdlThreadCheckStop()) {
-			tv.tv_sec = 60;		/* wake up every second if no input */
-			tv.tv_usec = 0;
-			retval = CtdlThreadSelect(highest + 1, &master_fds, NULL, NULL, &tv);
-		}
-		else
-			return NULL;
-
-		/* Now figure out who made this select() unblock.
-		 * First, check for an error or exit condition.
-		 */
-		if (retval < 0) {
-			if (errno == EBADF) {
-				CtdlLogPrintf(CTDL_NOTICE, "select() failed: (%s)\n",
-					strerror(errno));
-				continue;
-			}
-			if (errno != EINTR) {
-				CtdlLogPrintf(CTDL_EMERG, "Exiting (%s)\n", strerror(errno));
-				CtdlThreadStopAll();
-			} else {
-				CtdlLogPrintf(CTDL_DEBUG, "Interrupted CtdlThreadSelect.\n");
-				if (CtdlThreadCheckStop()) return(NULL);
-				continue;
-			}
-		}
-		else if(retval == 0) {
-			if (CtdlThreadCheckStop()) return(NULL);
-			continue;
-		}
-		/* Next, check to see if it's a new client connecting
-		 * on a master socket.
-		 */
-		else for (serviceptr = ServiceHookTable; serviceptr != NULL;
-		     serviceptr = serviceptr->next ) {
-
-			if (FD_ISSET(serviceptr->msock, &master_fds)) {
-				ssock = accept(serviceptr->msock, NULL, 0);
-				if (ssock >= 0) {
-					CtdlLogPrintf(CTDL_DEBUG,
-						"New client socket %d\n",
-						ssock);
-
-					/* The master socket is non-blocking but the client
-					 * sockets need to be blocking, otherwise certain
-					 * operations barf on FreeBSD.  Not a fatal error.
-					 */
-					if (fcntl(ssock, F_SETFL, 0) < 0) {
-						CtdlLogPrintf(CTDL_EMERG,
-							"citserver: Can't set socket to blocking: %s\n",
-							strerror(errno));
-					}
-
-					/* New context will be created already
-					 * set up in the CON_EXECUTING state.
-					 */
-					con = CreateNewContext();
-
-					/* Assign our new socket number to it. */
-					con->client_socket = ssock;
-					con->h_command_function =
-						serviceptr->h_command_function;
-					con->h_async_function =
-						serviceptr->h_async_function;
-					con->h_greeting_function = serviceptr->h_greeting_function;
-					con->ServiceName =
-						serviceptr->ServiceName;
-					
-					/* Determine whether it's a local socket */
-					if (serviceptr->sockpath != NULL)
-						con->is_local_socket = 1;
-	
-					/* Set the SO_REUSEADDR socket option */
-					i = 1;
-					setsockopt(ssock, SOL_SOCKET,
-						SO_REUSEADDR,
-						&i, sizeof(i));
-
-					con->state = CON_GREETING;
-
-					retval--;
-					if (retval == 0)
-						break;
-				}
-			}
-		}
-	}
-	CtdlClearSystemContext();
-
-	return NULL;
-}
 
 
 
@@ -1531,115 +1404,5 @@ int SyslogFacility(char *name)
 		if(!strcasecmp(name, facTbl[i].name))
 			return facTbl[i].facility;
 	}
-	enable_syslog = 0;
 	return LOG_DAEMON;
 }
-
-
-/********** MEM CHEQQER ***********/
-
-#ifdef DEBUG_MEMORY_LEAKS
-
-#undef malloc
-#undef realloc
-#undef strdup
-#undef free
-
-void *tracked_malloc(size_t size, char *file, int line) {
-	struct igheap *thisheap;
-	void *block;
-
-	block = malloc(size);
-	if (block == NULL) return(block);
-
-	thisheap = malloc(sizeof(struct igheap));
-	if (thisheap == NULL) {
-		free(block);
-		return(NULL);
-	}
-
-	thisheap->block = block;
-	strcpy(thisheap->file, file);
-	thisheap->line = line;
-	
-	begin_critical_section(S_DEBUGMEMLEAKS);
-	thisheap->next = igheap;
-	igheap = thisheap;
-	end_critical_section(S_DEBUGMEMLEAKS);
-
-	return(block);
-}
-
-
-void *tracked_realloc(void *ptr, size_t size, char *file, int line) {
-	struct igheap *thisheap;
-	void *block;
-
-	block = realloc(ptr, size);
-	if (block == NULL) return(block);
-
-	thisheap = malloc(sizeof(struct igheap));
-	if (thisheap == NULL) {
-		free(block);
-		return(NULL);
-	}
-
-	thisheap->block = block;
-	strcpy(thisheap->file, file);
-	thisheap->line = line;
-	
-	begin_critical_section(S_DEBUGMEMLEAKS);
-	thisheap->next = igheap;
-	igheap = thisheap;
-	end_critical_section(S_DEBUGMEMLEAKS);
-
-	return(block);
-}
-
-
-
-void tracked_free(void *ptr) {
-	struct igheap *thisheap;
-	struct igheap *trash;
-
-	free(ptr);
-
-	if (igheap == NULL) return;
-	begin_critical_section(S_DEBUGMEMLEAKS);
-	for (thisheap = igheap; thisheap != NULL; thisheap = thisheap->next) {
-		if (thisheap->next != NULL) {
-			if (thisheap->next->block == ptr) {
-				trash = thisheap->next;
-				thisheap->next = thisheap->next->next;
-				free(trash);
-			}
-		}
-	}
-	if (igheap->block == ptr) {
-		trash = igheap;
-		igheap = igheap->next;
-		free(trash);
-	}
-	end_critical_section(S_DEBUGMEMLEAKS);
-}
-
-char *tracked_strdup(const char *s, char *file, int line) {
-	char *ptr;
-
-	if (s == NULL) return(NULL);
-	ptr = tracked_malloc(strlen(s) + 1, file, line);
-	if (ptr == NULL) return(NULL);
-	strncpy(ptr, s, strlen(s));
-	return(ptr);
-}
-
-void dump_heap(void) {
-	struct igheap *thisheap;
-
-	for (thisheap = igheap; thisheap != NULL; thisheap = thisheap->next) {
-		CtdlLogPrintf(CTDL_CRIT, "UNFREED: %30s : %d\n",
-			thisheap->file, thisheap->line);
-	}
-}
-
-#endif /*  DEBUG_MEMORY_LEAKS */
